Option Strict On

Imports system.Globalization
Imports System.Threading
Imports System.Resources
Imports System.ComponentModel
Imports System.Runtime.InteropServices

<ClassInterface(ClassInterfaceType.None)> _
Public Class BrowseForFolder
    Inherits System.ComponentModel.Component

#Region " Component Designer generated Code "

    Public Sub New(ByVal Container As System.ComponentModel.IContainer)
        MyClass.New()

        'Fr Support von Windows.Forms-Klassenkompositions-Designer
        Container.Add(Me)
    End Sub

    Public Sub New()
        MyBase.New()

        ' Dieser Aufruf ist fr den Komponenten-Designer erforderlich.
        InitializeComponent()

        ' Initialisierungen nach dem Aufruf InitializeComponent() hinzufgen
        Initialize()
    End Sub

    'Die Komponente berschreibt den Lschvorgang zum Bereinigen der Komponentenliste.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    ' Fr Komponenten-Designer erforderlich
    Private components As System.ComponentModel.IContainer

    'HINWEIS: Die folgende Prozedur ist fr den Komponenten-Designer erforderlich
    'Sie kann mit dem Komponenten-Designer modifiziert werden.
    'Verwenden Sie nicht den Code-Editor zur Bearbeitung.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container
    End Sub

#End Region

#Region " Fields and Properties "

    'Controls
    Private WithEvents browse As Windows.Forms.Form
    Private WithEvents treeView As ExpTree
    Private WithEvents m_OK As Button
    Private WithEvents m_cancel As Button
    Private WithEvents m_newFolder As Button

    Private m_selectedPath As String
    ''' <summary>
    ''' Currently Selected Path
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue("")> _
    Public Property SelectedPath() As String
        Get
            If IsNothing(Me.treeView.SelectedItem) Then Return ""
            Dim path As String = Me.treeView.SelectedItem.Path
            If path.StartsWith("::") Then
                Return Me.treeView.SelectedItem.DisplayName
            Else
                Return path
            End If
        End Get
        Set(ByVal Value As String)
            treeView.ExpandANode(Value)
        End Set
    End Property

    ''' <summary>
    ''' Title for the Dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Title for the Dialog"), _
    DefaultValue("Browse For Folder")> _
    Public Property Title() As String
        Get
            Return Me.browse.Text
        End Get
        Set(ByVal Value As String)
            Me.browse.Text = Value
        End Set
    End Property

    Private m_allowNewFolders As Boolean = True
    ''' <summary>
    ''' Include the New Folder button in the dialog box
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Include the New Folder button in the dialog box."), _
    DefaultValue(True)> _
    Public Property AllowNewFolders() As Boolean
        Get
            Return m_allowNewFolders
        End Get
        Set(ByVal Value As Boolean)
            m_allowNewFolders = Value
        End Set
    End Property

    ''' <summary>
    ''' Root directory for the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Root directory for the control."), _
    DefaultValue(GetType(ExpTree.StartDir), "Desktop")> _
    Public Property RootDirectory() As ExpTree.StartDir
        Get
            Return Me.treeView.StartUpDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            Me.treeView.StartUpDirectory = Value
        End Set
    End Property

    Private m_folder As CShItem
    ''' <summary>
    ''' Last selected folder
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property Folder() As CShItem
        Get
            Return m_folder
        End Get
    End Property

    ''' <summary>
    ''' Initial location for the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Initial location for the dialog")> _
    Public Property StartLocation() As FormStartPosition
        Get
            Return Me.browse.StartPosition
        End Get
        Set(ByVal Value As FormStartPosition)
            Me.browse.StartPosition = Value
        End Set
    End Property

    Private m_topMost As Boolean = False
    ''' <summary>
    ''' Controls whether the dialog will always be the top window - use with TopMost Forms
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Controls whether the dialog will always be the top window - use with TopMost Forms"), _
    DefaultValue(False)> _
    Public Property TopMost() As Boolean
        Get
            Return m_topMost
        End Get
        Set(ByVal Value As Boolean)
            m_topMost = Value
        End Set
    End Property

#End Region

#Region " Form "

    Private Sub Initialize()
        'Create Form
        Me.browse = New Windows.Forms.Form
        Me.browse.TopMost = m_topMost
        Me.browse.Icon = GetIcon()
        Me.browse.Size = New Size(280, 360)
        Me.browse.MinimumSize = New Size(280, 360)
        Me.browse.Text = My.Resources.ControlStrings.BrowseForFolder
        Me.browse.StartPosition = FormStartPosition.Manual
        Me.treeView = New ExpTree
        Me.treeView.Size = New Size(263, 288)
        Me.treeView.StartUpDirectory = ExpTree.StartDir.Desktop
        Me.treeView.Anchor = (((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right)
        Me.treeView.Location = New Point(5, 5)
        Me.treeView.TabIndex = 0
        Me.treeView.AllowFolderRename = True
        Dim g As Graphics = Me.browse.CreateGraphics
        Dim stringSize As SizeF = _
            g.MeasureString(My.Resources.ControlStrings.Cancel, Me.browse.Font)
        Dim intWidth As Integer = CInt(stringSize.Width)
        Dim buttonSize As New Size(intWidth + 10, 22)
        Me.m_OK = New Button
        Me.m_OK.FlatStyle = FlatStyle.System
        Me.m_OK.DialogResult = DialogResult.OK
        Me.m_OK.Size = buttonSize
        Me.m_OK.Text = My.Resources.ControlStrings.OK
        Me.m_OK.Location = New Point(262 - buttonSize.Width, 297)
        Me.m_OK.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Right)
        Me.m_OK.TabIndex = 3
        Me.m_OK.Enabled = False
        Me.m_cancel = New Button
        Me.m_cancel.FlatStyle = FlatStyle.System
        Me.m_cancel.DialogResult = DialogResult.Cancel
        Me.m_cancel.Size = buttonSize
        Me.m_cancel.Text = My.Resources.ControlStrings.Cancel
        Me.m_cancel.Location = New Point(m_OK.Left - 5 - buttonSize.Width, 297)
        Me.m_cancel.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Right)
        Me.m_cancel.TabIndex = 2
        Me.m_newFolder = New Button
        Me.m_newFolder.FlatStyle = FlatStyle.System
        stringSize = _
            g.MeasureString(My.Resources.ControlStrings.NewFolder, Me.browse.Font)
        intWidth = CInt(stringSize.Width)
        Me.m_newFolder.Size = New Size(intWidth + 10, 22)
        Me.m_newFolder.Text = My.Resources.ControlStrings.NewFolder
        Me.m_newFolder.Location = New Point(5, 297)
        Me.m_newFolder.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Left)
        Me.m_newFolder.TabIndex = 1
        Me.m_newFolder.Enabled = False
        Me.browse.Controls.AddRange(New Control() {Me.treeView, Me.m_OK, Me.m_cancel, m_newFolder})

    End Sub

    Private Function GetIcon() As Icon
        'Returns embedded Icon from Assembly
        Dim theAssembly As System.Reflection.Assembly
        theAssembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim assemblyName As String = theAssembly.GetName().Name
        Dim resourceName As String
        Dim iconStream As System.IO.Stream
        Dim theIcon As Icon
        resourceName = "ExplorerControls.browse.ico"
        iconStream = theAssembly.GetManifestResourceStream(resourceName)
        theIcon = New Icon(iconStream)
        Return theIcon
    End Function

#End Region

#Region " Methods "

    ''' <summary>
    ''' Display the Dialog
    ''' </summary>
    ''' <returns>A member of the Windows.Forms.DialogResult Enum</returns>
    ''' <remarks></remarks>
    Public Function ShowDialog() As DialogResult
        'Show form
        Me.browse.TopMost = m_topMost
        If m_allowNewFolders Then
            m_newFolder.Visible = True
        Else
            m_newFolder.Visible = False
        End If
        ShowDialog = Me.browse.ShowDialog()
    End Function

    Private Sub m_OK_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_OK.Click
        'Set "Folder" to selected folder 
        m_folder = Me.treeView.SelectedItem
    End Sub

    Private Sub m_makeNewFolder_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_newFolder.Click
        'Get new folder name
        Dim name As String = My.Resources.ControlStrings.NewFolder
        Dim n As Integer
        Dim path As String = treeView.SelectedItem.Path
        Dim newPath As String
        Dim tempPath As String = ""

        If path.Length = 3 Then
            newPath = path & name
        Else
            newPath = path & "\" & name
        End If

        Try
            For n = 1 To 10000
                If IO.Directory.Exists(newPath & tempPath) Then
                    tempPath = "(" & n & ")"
                Else
                    Exit For
                End If
                If n = 10000 Then
                    Exit Sub
                End If
            Next
            newPath = newPath & tempPath
            If Not IO.Directory.Exists(newPath) Then
                IO.Directory.CreateDirectory(newPath)
                'Select newly added folder
                treeView.RefreshTree()
                treeView.ExpandANode(newPath)
                treeView.tv1.SelectedNode.BeginEdit()
            Else
                MessageBox.Show(My.Resources.ControlStrings.CouldNotCreateFolder & vbCrLf & newPath, _
                         My.Resources.ControlStrings._Error, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, _
                         My.Resources.ControlStrings._Error, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub treeView_ExpTreeNodeSelected(ByVal SelPath As String, ByVal Item As CShItem) Handles treeView.ExpTreeNodeSelected
        If Not Me.browse.Visible Then Exit Sub
        'Enable/Disable m_makeNewFolder if a Special 
        'Folder or a SharePoint Folder is selected (The IsLink is for
        'Network Places).
        If Item.IsLink Or Item.Path.StartsWith("::") Then
            m_OK.Enabled = False
            m_newFolder.Enabled = False
        Else
            m_OK.Enabled = True
            If Item.IsSharePoint Then
                m_newFolder.Enabled = False
            Else
                m_newFolder.Enabled = True
            End If
        End If
    End Sub

#End Region

End Class

