Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports System.Drawing.Design
Imports Microsoft.Win32

<ClassInterface(ClassInterfaceType.None)> _
Public Class SaveFileDialog
    Inherits System.ComponentModel.Component

#Region " Fields and Properties "

    Private m_filedialog As New FileDialog

    ''' <summary>
    ''' Filters the filetypes shown in the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Filters the filetypes shown in the dialog"), _
    DefaultValue("")> _
       Public Property Filter() As String
        Get
            Return m_filedialog.Filter
        End Get
        Set(ByVal Value As String)
            m_filedialog.Filter = Value
        End Set
    End Property

    ''' <summary>
    ''' Zero-based index for the selected filter
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Zero-based index for the selected filter"), _
    DefaultValue(-1)> _
    Public Property SelectedFilterIndex() As Integer
        Get
            Return m_filedialog.SelectedFilterIndex
        End Get
        Set(ByVal Value As Integer)
            m_filedialog.SelectedFilterIndex = Value
        End Set
    End Property

    ''' <summary>
    ''' The selected file name from the Dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), Description("The selected file name from the Dialog")> _
    Public ReadOnly Property FileName() As String
        Get
            Return m_filedialog.FileName
        End Get
    End Property

    ''' <summary>
    ''' The startup folder for the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("The startup folder for the dialog."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property StartUpDirectory() As ExpTree.StartDir
        Get
            Return m_filedialog.StartUpDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            m_filedialog.StartUpDirectory = Value
        End Set
    End Property

    ''' <summary>
    ''' The root folder for the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("The root folder for the dialog."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property RootDirectory() As ExpTree.StartDir
        Get
            Return m_filedialog.RootDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            m_filedialog.RootDirectory = Value
        End Set
    End Property

    ''' <summary>
    ''' Controls whether the dialog will always be the top window - use with TopMost Forms
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Controls whether the dialog will always be the top window - use with TopMost Forms"), _
    DefaultValue(False)> _
    Public Property TopMost() As Boolean
        Get
            Return m_filedialog.TopMost
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.TopMost = Value
        End Set
    End Property

    ''' <summary>
    ''' Add hidden files to the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Add hidden files to the dialog."), _
    DefaultValue(True)> _
    Public Property ShowHiddenFiles() As Boolean
        Get
            Return m_filedialog.ShowHiddenFiles
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.ShowHiddenFiles = Value
        End Set
    End Property

    ''' <summary>
    ''' Whether to append a file extension if none is given
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>This will append the first extension in the active filter array</remarks>
    <Category("Behavior"), Description("Whether to append a file extension if none is given. This will append " _
        & "the first extension in the active filter array."), DefaultValue(True)> _
    Public Property Append() As Boolean
        Get
            Return m_filedialog.Append
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.Append = Value
        End Set
    End Property

    ''' <summary>
    ''' The ExpListColumnCollection to be used with the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior")> _
    <Editor(GetType(System.ComponentModel.Design.CollectionEditor), GetType(UITypeEditor))> _
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Content)> _
    Public ReadOnly Property Columns() As ExpList.ExpListColumnCollection
        Get
            Return m_filedialog.browser.Columns
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the current view for the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DisplayName("View"), Category("Appearance"), EditorBrowsable(EditorBrowsableState.Never), _
    Description("Gets or sets the current view for the dialog"), DefaultValue(GetType(ExpListView), "LargeIcon")> _
    Public Property ViewStyle() As ExpListView
        Get
            Return m_filedialog.browser.View
        End Get
        Set(ByVal value As ExpListView)
            m_filedialog.browser.View = value
        End Set
    End Property

    ''' <summary>
    ''' Allows the user to change the available columns by activating the ColumnHeaderContextMenu
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to change the available columns by activating the ColumnHeaderContextMenu."), _
    DefaultValue(True)> _
    Public Property AllowColumnChanges() As Boolean
        Get
            Return m_filedialog.browser.AllowColumnChanges
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.browser.AllowColumnChanges = Value
        End Set
    End Property

    ''' <summary>
    ''' Allow the user to rename files and folders
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to rename files and folders."), _
    DefaultValue(True)> _
    Public Property AllowRename() As Boolean
        Get
            Return m_filedialog.browser.AllowRename
        End Get
        Set(ByVal value As Boolean)
            m_filedialog.browser.AllowRename = value
        End Set
    End Property

    ''' <summary>
    ''' Allow the user to create new folders
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to create new folders."), _
    DefaultValue(True)> _
        Public Property AllowNewFolders() As Boolean
        Get
            Return m_filedialog.browser.AllowNewFolders
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.browser.AllowNewFolders = Value
        End Set
    End Property

    ''' <summary>
    ''' Determines whether ExpList columns can be reordered by the user
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Determines whether ExpList columns can be reordered by the user."), _
    DefaultValue(True)> _
    Public Property AllowColumnReorder() As Boolean
        Get
            Return m_filedialog.browser.AllowColumnReorder
        End Get
        Set(ByVal Value As Boolean)
            m_filedialog.browser.AllowColumnReorder = Value
        End Set
    End Property

    ''' <summary>
    ''' Initial location for the dialog
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Options"), Description("Initial location for the dialog")> _
    Public Property StartLocation() As FormStartPosition
        Get
            Return m_filedialog.StartPosition
        End Get
        Set(ByVal Value As FormStartPosition)
            m_filedialog.StartPosition = Value
        End Set
    End Property

#End Region

#Region " Component Designer generated code "

    Public Sub New(ByVal Container As System.ComponentModel.IContainer)
        MyClass.New()

        'Required for Windows.Forms Class Composition Designer support
        Container.Add(Me)
    End Sub

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call
        m_filedialog.Type = DialogType.Save
    End Sub

    'Component overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Component Designer
    'It can be modified using the Component Designer.
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container
    End Sub

#End Region

#Region " Public Methods "

    ''' <summary>
    ''' Displays the dialog
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ShowDialog() As DialogResult
        'Show the Form
        Return m_filedialog.ShowDialog()
    End Function

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <param name="xmlFile">Xml file for writing the DataSet to</param>
    ''' <remarks></remarks>
    Public Overloads Sub SaveState(ByVal xmlFile As String)
        Dim ds As DataSet = SaveState()
        ds.WriteXml(xmlFile)
    End Sub

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <returns>DataSet with the current settings</returns>
    ''' <remarks></remarks>
    Public Overloads Function SaveState() As DataSet
        Return m_filedialog.SaveState()
    End Function

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <param name="regKey">Registry Key for saving the state</param>
    ''' <remarks></remarks>
    Public Overloads Sub SaveState(ByVal regKey As RegistryKey)
        m_filedialog.SaveState(regKey)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="xmlFile">Xml file with previously saved state information</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub LoadState(ByVal xmlFile As String)
        m_filedialog.LoadState(xmlFile)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="savedState">A DataSet with the previously saved state</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub LoadState(ByVal savedState As DataSet)
        LoadState(savedState, False)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state
    ''' </summary>
    ''' <param name="savedState">A DataSet with the previously saved state</param>
    ''' <param name="throwOnError">Determines if any errors will be caught or if they are rethrown to the calling Class</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub LoadState(ByVal savedState As DataSet, ByVal throwOnError As Boolean)
        m_filedialog.LoadState(savedState, throwOnError)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="savedState">A Registry Key with the previously saved state</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub LoadState(ByVal savedState As RegistryKey)
        LoadState(savedState, False)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state
    ''' </summary>
    ''' <param name="savedState">A Registry Key with the previously saved state</param>
    ''' <param name="throwOnError">Determines if any errors will be caught or if they are rethrown to the calling Class</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub LoadState(ByVal savedState As RegistryKey, ByVal throwOnError As Boolean)
        m_filedialog.LoadState(savedState, throwOnError)
    End Sub

#End Region

End Class
