Option Strict On

Imports System.Runtime.InteropServices

<ClassInterface(ClassInterfaceType.None)> _
Public Class ExpComboItem
    Implements IComparable

#Region " Fields and Properties "

    Friend startUpItem As Boolean = False

    Private m_item As CShItem
    ''' <summary>
    ''' The CShItem associated with this ExpComboItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property CShItem() As CShItem
        Get
            Return m_item
        End Get
    End Property

    Private m_text As String
    ''' <summary>
    ''' The text displayed for this item
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Text() As String
        Get
            Return m_text
        End Get
    End Property

    Private m_imageIndex As Integer
    ''' <summary>
    ''' The index of this items icon in the system image list
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ImageIndex() As Integer
        Get
            Return m_imageIndex
        End Get
    End Property

    Private m_offset As Integer = 0
    ''' <summary>
    ''' Offset from left side of dropdown list to show folder hierachy
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Friend ReadOnly Property Offset() As Integer
        Get
            Return m_offset
        End Get
    End Property

    Private m_children As New ArrayList
    ''' <summary>
    ''' The subfolders for the CShItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Children() As ArrayList
        Get
            Return m_children
        End Get
    End Property

    Private m_parent As ExpComboItem
    ''' <summary>
    ''' This items parent
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Parent() As ExpComboItem
        Get
            Return m_parent
        End Get
    End Property

#End Region

#Region " Constructors "

    ''' <summary>
    ''' Required for OnDrawItem method in ExpCombo Class and Com Interop
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

    End Sub

    ''' <summary>
    ''' Creates a new ExpComboItem
    ''' </summary>
    ''' <param name="item">The CShItem to be associated with this item</param>
    ''' <param name="parent">The parent item (Nothing for the root item)</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal item As CShItem, ByVal parent As ExpComboItem)
        'Do not add the item to the Children Collection of the parent here
        'this gets done in the AddItem Method of the ComboBox, if we do it
        'here then we don't know which items have already been added
        'to the ComboBox
        m_text = item.DisplayName
        m_imageIndex = SystemImageListManager.GetIconIndex(item)
        m_item = item
        If IsNothing(parent) Then
            m_offset = 0
        Else
            m_offset = parent.Offset + 1
            m_parent = parent
        End If
    End Sub

#End Region

#Region " Overrides "

    Public Overrides Function ToString() As String
        Return m_text
    End Function

#End Region

#Region " IComparable "

    Public Overridable Overloads Function CompareTo(ByVal obj As Object) As Integer _
               Implements IComparable.CompareTo
        If IsNothing(obj) Then Return 1 'non-existant is always low

        Return Me.CShItem.CompareTo(DirectCast(obj, ExpComboItem).CShItem)
    End Function

#End Region

End Class

