Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports System.Globalization
Imports System.Threading
Imports System.Text
Imports System.ComponentModel.Design
Imports System.Drawing.Design

''' <summary>
''' A ListView for displaying the file system
''' </summary>
''' <remarks></remarks>
<ClassInterface(ClassInterfaceType.None)> _
Public Class ExpList
    Inherits ListView

#Region " Events "

    'Declare the Delegates - this isn't necessary but it keeps everything in line with
    'other languages such as C#
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub FileActivateEventHandler(ByVal sender As Object, ByVal e As ExpListItemEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub FolderActivateEventHandler(ByVal sender As Object, ByVal e As ExpListItemEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub BeforeSortEventHandler(ByVal sender As Object, ByVal e As ExpListSortEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub AfterSortEventHandler(ByVal sender As Object, ByVal e As ExpListSortEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub GroupEventHandler(ByVal sender As Object, ByVal e As ExpListGroupEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub ItemsAddedEventHandler(ByVal sender As Object, ByVal e As ExpListItemsAddedEventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub NewFolderEventHandler(ByVal sender As Object, ByVal e As ExpListItemEventArgs)

    ''' <summary>
    ''' Fires when an ExpListItem that represents a file is activated
    ''' </summary>
    ''' <remarks></remarks>
    Public Event FileActivate As FileActivateEventHandler
    ''' <summary>
    ''' Fires when an ExpListItem that represents a folder is activated
    ''' </summary>
    ''' <remarks></remarks>
    Public Event FolderActivate As FolderActivateEventHandler
    ''' <summary>
    ''' Fires directly before the ExpList is sorted
    ''' </summary>
    ''' <remarks></remarks>
    Public Event BeforeSort As BeforeSortEventHandler
    ''' <summary>
    ''' Fires directly after the ExpList is sorted
    ''' </summary>
    ''' <remarks></remarks>
    Public Event AfterSort As AfterSortEventHandler
    ''' <summary>
    ''' Fires directly before the ExpList is grouped
    ''' </summary>
    ''' <remarks></remarks>
    Public Event Group As GroupEventHandler
    ''' <summary>
    ''' Fires directly after new Items have been added to the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Public Event ItemsAdded As ItemsAddedEventHandler
    ''' <summary>
    ''' If we don't post this Event the Browser (or any other Parent) will handle the Selected[Index/Node]Changed 
    ''' Event of an attached ExpCombo/ExpTree before the ExpList - i.e. before the Items have been added to the list.
    ''' </summary>
    ''' <remarks></remarks>
    Public Event SelectedFolderChanged As EventHandler
    ''' <summary>
    ''' Fires when a new folder is created in the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Public Event NewFolder As NewFolderEventHandler

#End Region

#Region " Fields and Properties "

#Region "       Shadows/Overrides "

    Private m_backgroundimage As String = ""
    ''' <summary>
    ''' Physical path of the background image to be used with the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("Physical path of the background image to be used."), _
    DefaultValue("")> _
    Public Shadows Property BackgroundImage() As String
        Get
            Return m_backgroundimage
        End Get
        Set(ByVal Value As String)
            SetListViewBackgroundImage(Value, m_backgroundImageAlignment)
            m_backgroundimage = Value
        End Set
    End Property

    Private m_backgroundImageAlignment As ContentAlignment = ContentAlignment.MiddleCenter
    ''' <summary>
    ''' Alignment of the background image
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("Alignment of the background image."), _
    DefaultValue(GetType(ContentAlignment), "MiddleCenter")> _
    Public Property BackgroundImagePosition() As ContentAlignment
        Get
            Return m_backgroundImageAlignment
        End Get
        Set(ByVal Value As ContentAlignment)
            m_backgroundImageAlignment = Value
            If m_backgroundimage.Length > 0 Then
                SetListViewBackgroundImage(m_backgroundimage, m_backgroundImageAlignment)
            End If
        End Set
    End Property

    Private m_columns As ExpListColumnCollection
    'I decided not to make this read only like the base class
    'because I thought it could be useful to be able to switch 
    'between column collections
    ''' <summary>
    ''' The ExpListColumnCollection to be used with the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior")> _
    <Editor(GetType(System.ComponentModel.Design.CollectionEditor), GetType(UITypeEditor))> _
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Content)> _
    Public Overridable Shadows Property Columns() As ExpListColumnCollection
        Get
            Return m_columns
        End Get
        Set(ByVal Value As ExpListColumnCollection)
            m_columns = Value
        End Set
    End Property

    Private m_items As ExpListItemCollection
    ''' <summary>
    ''' The items currently in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
   Public Overridable Shadows Property Items() As ExpListItemCollection
        Get
            Return m_items
        End Get
        Set(ByVal Value As ExpListItemCollection)
            m_items = Value
        End Set
    End Property

    Private m_selectedItems As SelectedExpListItemCollection
    ''' <summary>
    ''' The selected items in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
       Public Overridable Shadows ReadOnly Property SelectedItems() As SelectedExpListItemCollection
        Get
            Return m_selectedItems
        End Get
    End Property

    ''' <summary>
    ''' The Shortcut menu to display when the user right-clicks the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("The Shortcut menu to display when the user right-clicks the control")> _
    Public Overrides Property ContextMenuStrip() As System.Windows.Forms.ContextMenuStrip
        Get
            Return m_currentContextMenu
        End Get
        Set(ByVal Value As System.Windows.Forms.ContextMenuStrip)
            'Set the ContextMenu so it is stored in the
            'ContextMenuChanged Sub.
            m_currentContextMenu = Value
            'Activate the columnContextMenu
            If m_allowColumnChanges Then
                MyBase.ContextMenuStrip = columnContextMenu
            Else
                MyBase.ContextMenuStrip = m_currentContextMenu
            End If
        End Set
    End Property


    'Have to use a bit of trickery here - in VS2003 everything worked fine, but in 2005
    'the View property only works at runtime if you shadow it to use a different type
    Private m_view As ExpListView = ExpListView.LargeIcon
    ''' <summary>
    ''' Gets or sets the current view for the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DisplayName("View"), Category("Appearance"), EditorBrowsable(EditorBrowsableState.Never), _
    Description("Gets or sets the current view for the ExpList"), DefaultValue(GetType(ExpListView), "LargeIcon")> _
    Public Property ViewStyle() As ExpListView
        Get
            Return m_view
        End Get
        Set(ByVal value As ExpListView)
            Me.View = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the current view for the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public Shadows Property View() As ExpListView
        Get
            Return m_view
        End Get
        Set(ByVal value As ExpListView)
            'Store the TileSize if the current view is tiles
            If m_view = ExpListView.Tile Then m_tileSize = Me.TileSize
            m_view = value
            Select Case value
                Case ExpListView.Thumbnail
                    Me.OwnerDraw = True
                    MyBase.TileSize = New Size(126, (2 * (Me.Font.Height)) + 120)
                    If Me.Items.Count > 0 Then Me.RedrawItems(0, Me.Items.Count - 1, False)
                    MyBase.View = Windows.Forms.View.Tile
                Case ExpListView.Details
                    Me.TileSize = m_tileSize
                    Me.OwnerDraw = False
                    'TODO: Bug - the first time details is used the widths are reset when changing the view?!?!?
                    Dim columnWidths As New Hashtable
                    Dim col As ExpListColumn
                    For Each col In Columns
                        columnWidths.Add(col, col.Width)
                    Next
                    MyBase.View = CType(value, Windows.Forms.View)
                    For Each col In Columns
                        col.Width = CInt(columnWidths(col))
                    Next
                Case Else
                    Me.TileSize = m_tileSize
                    If Me.Items.Count > 0 Then Me.RedrawItems(0, Me.Items.Count - 1, False)
                    Me.OwnerDraw = False
                    MyBase.View = CType(value, Windows.Forms.View)
            End Select
        End Set
    End Property

    Private m_tileSize As Size = New Size(200, 80)
    ''' <summary>
    ''' Gets or sets the size of the tiles shown in tile view
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DefaultValue(GetType(Size), "200,80")> _
    Public Shadows Property TileSize() As Size
        Get
            Return m_tileSize
        End Get
        Set(ByVal value As Size)
            If Not m_view = ExpListView.Thumbnail Then
                MyBase.TileSize = value
                m_tileSize = value
            End If
        End Set
    End Property

#End Region

    Friend Shared testTime As New Date(1, 1, 1, 0, 0, 0)

    'For grouping
    Friend isMyComputer As Boolean

    'For context menu
    Private m_menuitems As ArrayList
    Private WithEvents m_currentContextMenu As ContextMenuStrip

    Friend WithEvents images As System.Windows.Forms.ImageList

    Private components As System.ComponentModel.IContainer

    'For Thumbnail LabelEdit
    Private WithEvents editTextBox As TextBox
    Private handled As Boolean = False

    'ContextMenu
    Friend WithEvents columnContextMenu As ContextMenuStrip
    Friend WithEvents sortMenu As ToolStripMenuItem
    Private WithEvents m_newFolder As ToolStripMenuItem
    Private WithEvents m_showInGroups As ToolStripMenuItem
    Private WithEvents m_viewMenu As ToolStripMenuItem
    Private WithEvents m_details As ToolStripMenuItem
    Private WithEvents m_list As ToolStripMenuItem
    Private WithEvents m_icons As ToolStripMenuItem
    Private WithEvents m_thumbnails As ToolStripMenuItem
    Private WithEvents m_tiles As ToolStripMenuItem
    Private WithEvents m_selectAll As ToolStripMenuItem
    Private WithEvents m_refresh As ToolStripMenuItem
    Private WithEvents m_remove As ToolStripMenuItem
    Private WithEvents m_removeAll As ToolStripMenuItem
    Private WithEvents m_open As ToolStripMenuItem
    Private WithEvents m_spacer1 As ToolStripSeparator
    Private WithEvents m_spacer2 As ToolStripSeparator
    Private WithEvents m_spacer3 As ToolStripSeparator
    Private WithEvents m_spacer4 As ToolStripSeparator

    Private WithEvents m_expCombo As ExpCombo
    ''' <summary>
    ''' The ExpCombo that controls the contents of the list
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Controls"), Description("Combo Box to determine the contents of the list."), _
    DefaultValue(CType(Nothing, Object))> _
    Public Property ExplorerCombo() As ExpCombo
        Get
            Return m_expCombo
        End Get
        Set(ByVal Value As ExpCombo)
            If Not IsNothing(Value) Then
                m_expCombo = Value
                'Reset the myComputer variable for grouping
                isMyComputer = False
                'Force the list to fill
                If m_expCombo.SelectedIndex > -1 Then
                    Fill(m_expCombo.SelectedItem.CShItem, True)
                End If
                m_expTree = Nothing
            End If
        End Set
    End Property

    Private WithEvents m_expTree As ExpTree
    ''' <summary>
    ''' The ExpTree that controls the contents of the list
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Controls"), Description("Tree view to determine the contents of the list."), _
    DefaultValue(CType(Nothing, Object))> _
    Public Property ExplorerTree() As ExpTree
        Get
            Return m_expTree
        End Get
        Set(ByVal Value As ExpTree)
            If Not IsNothing(Value) Then
                m_expTree = Value
                'Reset the myComputer variable for grouping
                isMyComputer = False
                'Force the list to fill
                If Not IsNothing(m_expTree.SelectedItem) Then
                    Fill(m_expTree.SelectedItem, True)
                End If
                m_expCombo = Nothing
            End If
        End Set
    End Property

    Private m_filter() As String
    ''' <summary>
    ''' Array list of file extensions allowed in the list
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Array list of file extensions allowed in the list."), _
    DefaultValue(CType(Nothing, Object))> _
    Public Property Filter() As String()
        Get
            Return m_filter
        End Get
        Set(ByVal Value As String())
            If IsNothing(Value) Then
                m_filter = Nothing
            ElseIf Value.Length = 0 Then
                m_filter = Nothing
            ElseIf (Value.Length = 1) And (Value(0).Trim = "" Or _
                    Value(0).Trim = "*" Or Value(0).Trim = "*.*") Then
                m_filter = Nothing
            Else
                m_filter = Value
            End If
        End Set
    End Property

    Private m_sortable As Boolean = True
    ''' <summary>
    ''' Determines if the list can be sorted by clicking the column headers
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Determines if the list can be sorted by clicking the column headers."), _
    DefaultValue(True)> _
    Public Property Sortable() As Boolean
        Get
            Return m_sortable
        End Get
        Set(ByVal Value As Boolean)
            m_sortable = Value
        End Set
    End Property

    Private m_sortOrder As System.Windows.Forms.SortOrder = SortOrder.None
    ''' <summary>
    ''' The SortOrder of the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SortOrder() As Windows.Forms.SortOrder
        Get
            Return m_sortOrder
        End Get
    End Property

    Private m_sortColumn As ExpListColumn
    ''' <summary>
    ''' The ExpListColumn currently used for sorting
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SortColumn() As ExpListColumn
        Get
            Return m_sortColumn
        End Get
    End Property

    ''' <summary>
    ''' Sets whether the ExpListItems should be grouped or not
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(False)> _
    Public Property Grouped() As Boolean
        Get
            Return m_showInGroups.Checked
        End Get
        Set(ByVal value As Boolean)
            If value Then
                GroupItems()
            Else
                ClearGroups()
            End If
        End Set
    End Property

    Private m_showHiddenFiles As Boolean = True
    ''' <summary>
    ''' Sets whether hidden files should be shown in the list or not
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Add hidden files to the list."), _
    DefaultValue(True)> _
    Public Property ShowHiddenFiles() As Boolean
        Get
            Return m_showHiddenFiles
        End Get
        Set(ByVal Value As Boolean)
            m_showHiddenFiles = Value
        End Set
    End Property

    Private m_language As Language = Language.OperatingSystem
    ''' <summary>
    ''' Sets the language for the ContextMenu and ToolTips
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(GetType(Language), "OperatingSystem")> _
    Public Property Language() As Language
        Get
            Return m_language
        End Get
        Set(ByVal Value As Language)
            If m_language = Value Then Exit Property
            m_language = Value
            Select Case Value
                'Set up the context menu and the columns with the appropriate culture
                Case Language.English
                    Thread.CurrentThread.CurrentUICulture = New CultureInfo("en")
                    SetText()
                Case Language.Deutsch
                    Thread.CurrentThread.CurrentUICulture = New CultureInfo("de")
                    SetText()
                Case Language.OperatingSystem
                    Thread.CurrentThread.CurrentUICulture = CultureInfo.CurrentCulture
                    SetText()
            End Select
        End Set
    End Property

    Private m_fileOpenOnActivate As Boolean = True
    ''' <summary>
    ''' Sets whether files should be opened on activation or if the 
    ''' FileActivated Event should be fired
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Open files when activated."), _
    DefaultValue(True)> _
        Public Property FileOpenOnActivate() As Boolean
        Get
            Return m_fileOpenOnActivate
        End Get
        Set(ByVal Value As Boolean)
            m_fileOpenOnActivate = Value
        End Set
    End Property

    Private m_folderOpenOnActivate As Boolean = True
    ''' <summary>
    ''' Sets whether folders should be opened on activation or if the 
    ''' FolderActivated Event should be fired
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Open folders when activated."), _
    DefaultValue(True)> _
        Public Property FolderOpenOnActivate() As Boolean
        Get
            Return m_folderOpenOnActivate
        End Get
        Set(ByVal Value As Boolean)
            m_folderOpenOnActivate = Value
        End Set
    End Property

    Private WithEvents m_defaultContextMenu As System.Windows.Forms.ContextMenuStrip
    ''' <summary>
    ''' The standard ContextMenu used with the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Controls"), Description("Default ContextMenuStrip for the ListView.")> _
    Public ReadOnly Property DefaultContextMenu() As ContextMenuStrip
        Get
            Return m_defaultContextMenu
        End Get
    End Property

    Private m_allowColumnChanges As Boolean = True
    ''' <summary>
    ''' Allows the user to change the available columns by activating the ColumnHeaderContextMenu
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to change the available columns by activating the ColumnHeaderContextMenu."), _
    DefaultValue(True)> _
    Public Property AllowColumnChanges() As Boolean
        Get
            Return m_allowColumnChanges
        End Get
        Set(ByVal Value As Boolean)
            m_allowColumnChanges = Value
            If Value Then
                MyBase.ContextMenuStrip = columnContextMenu
            Else
                MyBase.ContextMenuStrip = m_currentContextMenu
            End If
        End Set
    End Property

    Private m_allowRename As Boolean = True
    ''' <summary>
    ''' Allow the user to rename files and folders in the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to rename files and folders in the control."), _
    DefaultValue(True)> _
    Public Property AllowRename() As Boolean
        Get
            Return m_allowRename
        End Get
        Set(ByVal value As Boolean)
            m_allowRename = value
            MyBase.LabelEdit = value
        End Set
    End Property

    'Hide the Base Class LabelEdit Property
    Private Shadows ReadOnly Property LabelEdit() As Boolean
        Get
            Return m_allowRename
        End Get
    End Property

    Private m_allowNewFolders As Boolean = True
    ''' <summary>
    ''' Allow the user to create new folders in the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to create new folders in the control."), _
    DefaultValue(True)> _
        Public Property AllowNewFolders() As Boolean
        Get
            Return m_allowNewFolders
        End Get
        Set(ByVal Value As Boolean)
            m_allowNewFolders = Value
            m_newFolder.Enabled = Value
        End Set
    End Property

#End Region

#Region " Construction "

    ''' <summary>
    ''' Creates a new instance of the ExpList Class
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()

        'Initialise the column and items collections
        m_columns = New ExpListColumnCollection(Me)
        m_items = New ExpListItemCollection(Me)
        m_selectedItems = New SelectedExpListItemCollection(Me)

        'Initialize the ImageList and the context menus for the ColumnHeaders
        InitializeComponent()
        columnContextMenu = New ContextMenuStrip
        MyBase.ContextMenuStrip = columnContextMenu

        'Bind to the System Image List
        SystemImageListManager.SetListViewImageList(Me, True, False)
        SystemImageListManager.SetListViewImageList(Me, False, False)

        'Set ToolTips for items and sub-items
        Me.ShowItemToolTips = True
        SendMessage(Me.Handle, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_LABELTIP, LVS_EX_LABELTIP)

        'Allow Column reorder and LabelEdit by default
        Me.AllowColumnReorder = True
        MyBase.LabelEdit = True

        'For Thumbnail LabelEdit
        editTextBox = New TextBox
        editTextBox.BorderStyle = Windows.Forms.BorderStyle.FixedSingle
        editTextBox.Font = Me.Font
        editTextBox.Multiline = True
        editTextBox.TextAlign = HorizontalAlignment.Center
    End Sub

    'Clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.images = New System.Windows.Forms.ImageList(Me.components)
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(ExpList))
        Me.m_defaultContextMenu = New System.Windows.Forms.ContextMenuStrip(Me.components)
        Me.SuspendLayout()
        '
        'images
        '
        Me.images.ImageStream = CType(resources.GetObject("images.ImageStream"), System.Windows.Forms.ImageListStreamer)
        Me.images.TransparentColor = System.Drawing.Color.Transparent
        Me.images.Images.SetKeyName(0, "sort_up.gif")
        Me.images.Images.SetKeyName(1, "sort_down.gif")
        '
        'm_defaultContextMenu
        '
        m_showInGroups = New ToolStripMenuItem
        m_open = New ToolStripMenuItem
        m_newFolder = New ToolStripMenuItem
        m_spacer1 = New ToolStripSeparator
        m_spacer2 = New ToolStripSeparator
        m_spacer3 = New ToolStripSeparator
        m_spacer4 = New ToolStripSeparator
        m_selectAll = New ToolStripMenuItem
        m_removeAll = New ToolStripMenuItem
        m_remove = New ToolStripMenuItem
        m_refresh = New ToolStripMenuItem
        m_viewMenu = New ToolStripMenuItem
        m_details = New ToolStripMenuItem
        m_list = New ToolStripMenuItem
        m_icons = New ToolStripMenuItem
        m_thumbnails = New ToolStripMenuItem
        m_tiles = New ToolStripMenuItem
        sortMenu = New ToolStripMenuItem

        m_defaultContextMenu.Items.AddRange(New ToolStripItem() {m_open, m_newFolder, m_spacer1, m_remove, m_selectAll, m_removeAll, _
                m_spacer2, sortMenu, m_refresh, m_spacer3, m_viewMenu})

        'Set Text
        SetText()

        'Set Images
        m_open.Image = My.Resources.Images.Open
        m_newFolder.Image = My.Resources.Images.newFolder
        m_remove.Image = My.Resources.Images.delete
        m_refresh.Image = My.Resources.Images.refresh
        m_viewMenu.Image = My.Resources.Images.view_small
        m_details.Image = My.Resources.Images.details
        m_list.Image = My.Resources.Images.list
        m_icons.Image = My.Resources.Images.large_icon
        m_thumbnails.Image = My.Resources.Images.thumbnails
        m_tiles.Image = My.Resources.Images.tiles
        If IsNothing(m_sortColumn) Then m_showInGroups.Enabled = False

        m_viewMenu.DropDownItems.AddRange(New ToolStripMenuItem() {m_details, m_list, m_icons, m_thumbnails, m_tiles})
        If ShellDll.IsXpOrAbove Then
            sortMenu.DropDownItems.Add(m_spacer4)
            sortMenu.DropDownItems.Add(m_showInGroups)
        End If
        Me.ContextMenuStrip = m_defaultContextMenu
        '
        'ExpList
        '
        Me.ResumeLayout(False)

    End Sub

    Private Sub SetText()
        m_open.Text = My.Resources.ControlStrings.Open
        m_newFolder.Text = My.Resources.ControlStrings.NewFolder
        m_selectAll.Text = My.Resources.ControlStrings.SelectAll
        m_removeAll.Text = My.Resources.ControlStrings.RemoveAll
        m_remove.Text = My.Resources.ControlStrings.Remove
        m_refresh.Text = My.Resources.ControlStrings.Refresh
        m_viewMenu.Text = My.Resources.ControlStrings.View
        m_details.Text = My.Resources.ControlStrings.Details
        m_list.Text = My.Resources.ControlStrings.List
        m_icons.Text = My.Resources.ControlStrings.Icons
        m_thumbnails.Text = My.Resources.ControlStrings.Thumbnails
        m_tiles.Text = My.Resources.ControlStrings.Tiles
        sortMenu.Text = My.Resources.ControlStrings.SortBy
        m_showInGroups.Text = My.Resources.ControlStrings.ShowInGroups

        Dim itm As ToolStripMenuItem

        If columnContextMenu Is Nothing Then Exit Sub
        For Each itm In columnContextMenu.Items
            CType(itm.Tag, ExpListColumn).SetText()
        Next
    End Sub

#End Region

#Region " Methods "

#Region "       Shared Methods "

    ''' <summary>
    ''' Decodes special characters in a URL (e.g. replaces %20 with a space)
    ''' </summary>
    ''' <param name="url">The URL to be decoded</param>
    ''' <returns>The decoded URL</returns>
    ''' <remarks></remarks>
    Public Shared Function DecodeURL(ByVal url As String) As String
        'Return if there are no special characters 
        If url.LastIndexOf("%"c) = -1 Then Return url

        'Integer to hold character index
        Dim index As Integer
        'String to hold Hexidecimal character code
        Dim hexCharacter As String
        Do
            index = url.IndexOf("%"c)
            'Return if there are no more special characters
            If index = -1 Then Return url
            'The two characters after the '%' in a URL is the hexidecimal
            'ISO 8859 character code
            hexCharacter = "&H" & url.Substring(index + 1, 2)
            hexCharacter = Encoding.GetEncoding("ISO-8859-1").GetString(New Byte() {CByte(hexCharacter)})
            url = url.Replace("%" & url.Substring(index + 1, 2), hexCharacter)
        Loop

    End Function

#End Region

#Region "       TreeView & ComboBox "

    'Method to fill the list with all of the items in the currently selected 
    'folder of the attached TreeView or ComboBox. Can also be used directly
    'when the list is not attached to anything through the FillExpList Sub
    Private Sub Fill(ByVal CSI As CShItem, ByVal clearExistingItems As Boolean)
        Dim dirList As New ArrayList
        Dim fileList As New ArrayList
        Dim totalItems As Integer
        If CSI.Path = CShItem.GetCShItem(ShellDll.CSIDL.DRIVES).Path Then
            isMyComputer = True
            dirList = CSI.GetDirectories(False)  'avoid re-query since only has dirs
        Else
            isMyComputer = False
            dirList = CSI.GetDirectories(False)
            fileList = CSI.GetFiles()
        End If

        totalItems = dirList.Count + fileList.Count
        If totalItems > 0 Then

            dirList.Sort()
            fileList.Sort()
            Dim combList As New ArrayList(totalItems)
            combList.AddRange(dirList)
            combList.AddRange(fileList)

            If clearExistingItems Then Me.Items.Clear()
            Me.Items.AddRange(CType(combList.ToArray(GetType(CShItem)), CShItem()))
        Else
            If clearExistingItems Then Me.Items.Clear()
        End If
    End Sub

    ''' <summary>
    ''' NodeSelect Event handler for attached ExpTree
    ''' </summary>
    ''' <param name="SelPath">Path for the selected folder in the ExpTree</param>
    ''' <param name="Item">Selected Item in the ExpTree</param>
    ''' <remarks></remarks>
    Protected Overridable Sub ExpTree_NodeSelect(ByVal SelPath As String, ByVal Item As CShItem) Handles m_expTree.ExpTreeNodeSelected
        'I use Begin/EndUpdate here to allow Custom Columns to update any text
        'and re-sort before any Items are shown.
        Me.BeginUpdate()
        Fill(Item, True)
        'Post the SelectedFolderChanged Event
        OnSelectedFolderChanged(New EventArgs)
        Me.EndUpdate()
    End Sub

    ''' <summary>
    ''' SelectedIndexChanged Event handler for attached ExpCombo
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub ExpCombo_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_expCombo.SelectedIndexChanged
        If m_expCombo.SelectedIndex < 0 Then Exit Sub

        'I use Begin/EndUpdate here to allow Custom Columns to update any text
        'and re-sort before any Items are shown.
        Me.BeginUpdate()
        Fill(m_expCombo.SelectedItem.CShItem, True)
        'Post the SelectedFolderChanged Event
        OnSelectedFolderChanged(New EventArgs)
        Me.EndUpdate()
    End Sub

#End Region

#Region "       Adding Items "

#Region "           SetItemText "

    ''' <summary>
    ''' Sets the text for the specified ExpListItem
    ''' </summary>
    ''' <param name="item"></param>
    ''' <remarks></remarks>
    Protected Friend Overridable Overloads Sub SetItemText(ByVal item As ExpListItem)
        Dim column As ExpListColumn
        For Each column In Me.Columns
            SetItemText(item, column)
        Next
    End Sub

    ''' <summary>
    ''' Sets the specified column text for the specified ExpListItem
    ''' </summary>
    ''' <param name="item"></param>
    ''' <param name="column"></param>
    ''' <remarks></remarks>
    Protected Friend Overridable Overloads Sub SetItemText(ByVal item As ExpListItem, ByVal column As ExpListColumn)

        Dim index As Integer = column.Index
        If index < 0 Then Exit Sub

        'Make sure the subitem exists
        Do While item.SubItems.Count <= index
            item.SubItems.Add("")
        Loop
        'Set the text for the CShItem properties, just
        'ignore any custom columns.
        Dim columnType As ExpListColumnType = column.ColumnType
        Select Case columnType
            Case ExpListColumnType.Name
                item.SubItems(index).Text = item.CShItem.DisplayName

            Case ExpListColumnType.Size
                item.SubItems(index).Text = item.CShItem.Size

            Case ExpListColumnType.Type
                item.SubItems(index).Text = item.CShItem.TypeName

            Case ExpListColumnType.Path
                If item.CShItem.Path.StartsWith("::") Then
                    item.SubItems(index).Text = item.CShItem.DisplayName
                ElseIf item.CShItem.IsSharePoint Then
                    'For SharePoint decode the URL (remove %20 etc)
                    item.SubItems(index).Text = ExpList.DecodeURL(item.CShItem.Path)
                Else
                    item.SubItems(index).Text = item.CShItem.Path
                End If

            Case ExpListColumnType.Created
                'Ignore disks
                If item.CShItem.IsDisk Then
                    item.SubItems(index).Text = ""
                Else
                    '"#1/1/0001 12:00:00 AM#" is empty
                    If item.CShItem.CreationTime = testTime Then
                        item.SubItems(index).Text = ""
                    Else
                        item.SubItems(index).Text = CStr(item.CShItem.CreationTime)
                    End If
                End If

            Case ExpListColumnType.Modified
                If item.CShItem.IsDisk Then
                    item.SubItems(index).Text = ""
                Else
                    If item.CShItem.LastWriteTime = testTime Then
                        item.SubItems(index).Text = ""
                    Else
                        item.SubItems(index).Text = CStr(item.CShItem.LastWriteTime)
                    End If
                End If

        End Select
        item.ToolTipText = item.CShItem.DisplayName & ControlChars.CrLf _
                    & item.CShItem.TypeName & ControlChars.CrLf & item.CShItem.Size
    End Sub

#End Region

    ''' <summary>
    ''' Method for adding an entire folder to the list
    ''' </summary>
    ''' <param name="item">A CShItem representing a folder</param>
    ''' <remarks>This can only be used with an ExpList that is 
    ''' not attached to another control.</remarks>
    Public Sub FillExpList(ByVal item As CShItem, ByVal clearExistingItems As Boolean)
        If Not item.IsFolder Then
            Throw New ApplicationException("Invalid CShItem passed")
        End If
        If IsNothing(m_expCombo) And IsNothing(m_expTree) Then
            Fill(item, clearExistingItems)
        End If
    End Sub

    ''' <summary>
    ''' Checks a CShItem against the ExpLists Filter property
    ''' </summary>
    ''' <param name="item">The CShItem to be checked</param>
    ''' <returns>Returns True if the item can be added, otherwise False</returns>
    ''' <remarks>This Function does not add the CShItem to the list, 
    ''' it only checks if the CShItem can be added with the current Filter property</remarks>
    Public Function FilterItem(ByVal item As CShItem) As Boolean
        'Returns true if file extension is in the m_filter array
        If IsNothing(m_filter) Then
            Return True
        End If
        Dim count As Integer
        Dim fileName As String = item.GetFileName
        For count = 0 To m_filter.Length - 1
            'Check the extension against the characters after the point in the filter string
            If fileName.ToLower.Substring(fileName.LastIndexOf("."c) + 1) = _
                m_filter(count).ToLower.Substring(m_filter(count).LastIndexOf("."c) + 1).Trim Then
                Return True
                Exit Function
            End If
        Next
        Return False
    End Function

#End Region

#Region "       Opening Items "

    ''' <summary>
    ''' Item Activate Event Handler for the ExpList
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub ExpList_ItemActivate(ByVal sender As System.Object, ByVal e As System.EventArgs) _
        Handles MyBase.ItemActivate

        'Check to see if we should raise an event or just open the item
        Dim shell As CShItem = Me.SelectedItems.Item(0).CShItem
        If shell.IsFolder Then
            If Not m_folderOpenOnActivate Then
                OnFolderActivate(New ExpListItemEventArgs(Me.SelectedItems.Item(0)))
                Exit Sub
            End If
        Else
            If Not m_fileOpenOnActivate Then
                OnFileActivate(New ExpListItemEventArgs(Me.SelectedItems.Item(0)))
                Exit Sub
            End If
        End If

        'Click
        Sounds.Click()

        'Open the item
        OpenSelectedListItems()

    End Sub

    ''' <summary>
    ''' Opens the currently selected items in the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub OpenSelectedListItems()

        'Opens a folder or executes a file
        Dim listItem As ExpListItem
        Dim itm As CShItem
        For Each listItem In Me.SelectedItems
            itm = listItem.CShItem
            If itm.IsFolder Then
                If Not IsNothing(m_expTree) Then
                    'Expands the related tree node
                    Me.ExplorerTree.ExpandANode(itm)
                ElseIf Not IsNothing(m_expCombo) Then
                    'Select the appropriate folder in 
                    'the attached ExpCombo
                    m_expCombo.SelectFolder(itm)
                    Debug.WriteLine(itm.Path)
                Else
                    'replaces items with the items in the folder.                    
                    Me.Items.Clear()
                    Me.Items.AddRange(CType(itm.GetItems.ToArray(GetType(CShItem)), CShItem()))
                End If
                'Only the first folder that is selected will 
                'be processed.
                Exit Sub
            Else
                'Open the selected files
                Try
                    If itm.Path.ToLower.EndsWith(".asm") Or _
                     itm.Path.ToLower.EndsWith(".dft") Or _
                     itm.Path.ToLower.EndsWith(".par") Or _
                     itm.Path.ToLower.EndsWith(".psm") Or _
                     itm.Path.ToLower.EndsWith(".pwd") Then
                        Dim solidEdge As Object
                        Try
                            solidEdge = GetObject(, "SolidEdge.Application")
                        Catch ex As Exception
                            solidEdge = CreateObject("SolidEdge.Application")
                            solidEdge.Visible = True
                            solidEdge.WindowState = 2
                        End Try
                        solidEdge.Documents.Open(itm.Path)
                    Else
                        System.Diagnostics.Process.Start(itm.Path)
                    End If
                Catch
                    'Ignore any Exceptions
                End Try
            End If
        Next

    End Sub

#End Region

#Region "       Refresh "

    ''' <summary>
    ''' Refreshes the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub RefreshList()
        If Not IsNothing(Me.ExplorerTree) Then
            Me.ExplorerTree.RefreshTree()
        ElseIf Not IsNothing(Me.ExplorerCombo) Then
            Me.ExplorerCombo.RefreshCombo()
            If Not IsNothing(m_expCombo.SelectedItem) Then
                Fill(m_expCombo.SelectedItem.CShItem, True)
            End If
        Else
            'Refreshes any items in the list to 
            'reflect any changes
            Dim item As ExpListItem
            For Each item In Me.Items
                Dim path As String = item.CShItem.Path
                item.CShItem = CShItem.GetCShItem(path)
            Next
        End If
    End Sub

    ''' <summary>
    ''' KeyUp Event Handler for the ExpList
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub ExpList_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyUp
        If e.KeyCode = Keys.F5 Then
            RefreshList()
        End If
    End Sub

#End Region

#Region "       Sorting "

    ''' <summary>
    ''' ColumnClick Event Handler for the ExpList
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub ExpList_ColumnClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ColumnClickEventArgs) _
        Handles MyBase.ColumnClick

        Me.Columns(e.Column).sortMenuItem.PerformClick()
    End Sub

    ''' <summary>
    ''' Sorts the items in the ExpList
    ''' </summary>
    ''' <param name="column">The ExpListColumn to sort by</param>
    ''' <param name="order">The order to sort</param>
    ''' <remarks></remarks>
    Public Overridable Sub SortByColumn(ByVal column As ExpListColumn, ByVal order As Windows.Forms.SortOrder)

        If m_sortable = False Then Exit Sub

        'Store the current cursor and show the hourglass
        Dim oldCursor As Cursor = Me.Parent.Cursor
        Dim hasParent As Boolean = Not IsNothing(Me.Parent)
        If hasParent Then Me.Parent.Cursor = Cursors.WaitCursor
        Me.BeginUpdate()

        'Assign flag for column number
        m_sortColumn = column
        'Store the new SortOrder
        m_sortOrder = order

        'Raise the BeforeSort Event to allow customisation
        'The calling Class can change the ExpColumns ListViewItemSorter here
        OnBeforeSort(New ExpListSortEventArgs(m_sortOrder, column))

        'Check the appropriate ContextMenu item
        Dim i As Integer
        For i = 0 To Me.Columns.Count - 1
            Me.Columns(i).sortMenuItem.Checked = False
        Next
        column.sortMenuItem.Checked = True

        'Enable the 'Show in Groups' menu item
        m_showInGroups.Enabled = True

        'Check if the list is already grouped 
        'If so then clear the grouping and
        'regroup and the end of the sub
        Dim grouped As Boolean = m_showInGroups.Checked
        If grouped Then ClearGroups()

        'Create a comparer.
        If IsNothing(column.ExpListItemSorter) Then
            Me.ListViewItemSorter = New _
                                 ExpListComparer(column, m_sortOrder, isMyComputer, grouped)
        Else
            Me.ListViewItemSorter = column.ExpListItemSorter
        End If

        'Sort.
        Me.Sort()

        'Regroup
        If grouped Then
            GroupItems()
        End If

        'Set ColumnHeader Icon
        SetSortIcon()

        'Set the Selected Column (Grey Background)
        SendMessage(Me.Handle, LVM_SETSELECTEDCOLUMN, column.Index, 0)

        'Raise the AfterSort Event
        OnAfterSort(New ExpListSortEventArgs(m_sortOrder, column))

        Me.EndUpdate()
        'Reset the cursor
        If hasParent Then Me.Parent.Cursor = oldCursor

    End Sub

    Private Sub SetSortIcon()
        If m_sortColumn Is Nothing Then Exit Sub

        'Add icons to the headers
        Dim hwnd As IntPtr
        Dim lret As IntPtr
        Dim col As LVCOLUMN
        Dim i As Integer

        Dim imgIndex As Integer
        Select Case m_sortOrder
            Case SortOrder.Ascending
                imgIndex = 0
            Case SortOrder.Descending
                imgIndex = 1
        End Select

        'Assign the ImageList to the header control.
        'The header control includes all columns.
        'Get a handle to the header control.
        hwnd = SendMessage(Me.Handle, LVM_GETHEADER, 0, 0)

        'Add the ImageList to the header control.
        lret = SendMessage(hwnd, HDM_SETIMAGELIST, 0, images.Handle)

        'Loop through the headers and set the image where appropriate
        For i = 0 To Me.Columns.Count - 1

            'Check to see the alignment of the Header
            Dim alignRight As Boolean
            If Me.Columns(i).TextAlign = HorizontalAlignment.Right Then
                alignRight = True
            Else
                alignRight = False
            End If

            'set up the Structure for the call to send Message
            col.mask = LVCF_FMT Or LVCF_IMAGE
            col.fmt = LVCF_STRING Or (CInt(alignRight) And LVCF_FMT)
            col.iImage = imgIndex
            col.cchTextMax = 0
            col.cx = 0
            col.iOrder = 0
            col.iSubItem = 0
            col.pszText = IntPtr.op_Explicit(0)
            'Set the image if this is the column that was clicked
            If i = m_sortColumn.Index Then
                col.fmt = col.fmt Or LVCFMT_IMAGE Or LVCF_BITMAP_ON_RIGHT
                'If you want the image to be on the left for right aligned
                'columns the use the following:
                'col.fmt = col.fmt Or LVCFMT_IMAGE Or (CInt(Not alignRight) And LVCF_BITMAP_ON_RIGHT)
            End If
            'Send the LVM_SETCOLUMN message.
            lret = SendMessage(Me.Handle, LVM_SETCOLUMN, i, col)
        Next
    End Sub

#End Region

#Region "       Drag and Drop "

    'I just added this feature for when I use the list as a selection list for batch processing
    'For this reason the files will not be written to the file system when the list is attached to
    'a TreeView or a ComboBox.
    Private Sub ExpList_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        'If list is bound to a TreeView or a ComboBox then don't let the user
        'add any new items
        If Not IsNothing(Me.ExplorerCombo) Or Not IsNothing(Me.ExplorerTree) Then Exit Sub
        'Adds files to ListView via DragDrop
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            Dim files() As String = CType(e.Data.GetData(DataFormats.FileDrop), String())
            Dim fileList As New ArrayList
            Dim count As Integer
            For count = 0 To files.Length - 1
                Me.Items.Add(files(count))
            Next
        Else
            If e.Data.GetDataPresent("ExplorerControls.ExpListItem", False) Then
                'Add a clone to the list
                Dim itm As ExpListItem
                itm = CType(CType(e.Data.GetData("ExplorerControls.ExpListItem"), ExpListItem).Clone, ExpListItem)
                Me.Items.Add(itm)
            End If
        End If
    End Sub

    Private Sub ExpList_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragEnter
        'If list is bound to a TreeView or a ComboBox then don't show 
        'any DragDropEffects
        If Not IsNothing(Me.ExplorerCombo) Or Not IsNothing(Me.ExplorerTree) Then Exit Sub
        'Shows the right icon when perfoming DragDrop
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            e.Effect = DragDropEffects.Copy
        ElseIf e.Data.GetDataPresent("ExplorerControls.ExpListItem", False) Then
            e.Effect = DragDropEffects.Copy
        End If
    End Sub

    Private Sub ExpList_ItemDrag(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemDragEventArgs) Handles Me.ItemDrag
        Dim itm As ExpListItem
        For Each itm In Me.SelectedItems
            DoDragDrop(itm, DragDropEffects.Copy)
        Next
    End Sub

#End Region

#Region "       Context Menu "

    'i have used the fact that the MouseDown Event isn't fired when the cursor
    'is hovering over a ColumnHeader to create a ContextMenu specifically for 
    'the column headers...

    Private Sub ExpList_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If m_allowColumnChanges And e.Button = Windows.Forms.MouseButtons.Right Then
            MyBase.ContextMenuStrip = m_currentContextMenu
        End If
    End Sub

    Private Sub m_currentContextMenu_Open(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles m_currentContextMenu.Opening

        If m_currentContextMenu Is m_defaultContextMenu Then
            m_details.Checked = False
            m_list.Checked = False
            m_icons.Checked = False
            m_thumbnails.Checked = False
            m_tiles.Checked = False

            Select Case Me.View
                Case ExpListView.Details
                    m_details.Checked = True
                Case ExpListView.LargeIcon
                    m_icons.Checked = True
                Case ExpListView.List
                    m_list.Checked = True
                Case ExpListView.Thumbnail
                    m_thumbnails.Checked = True
                Case ExpListView.Tile
                    m_tiles.Checked = True
            End Select
            'Enable removeAll, remove, selectAll and open etc...
            m_open.Visible = True
            m_open.Enabled = True
            m_removeAll.Visible = True
            m_removeAll.Enabled = True
            m_remove.Visible = True
            m_remove.Enabled = True
            m_selectAll.Visible = True
            m_selectAll.Enabled = True
            sortMenu.Visible = True
            sortMenu.Enabled = True
            m_newFolder.Visible = True
            m_spacer1.Visible = True

            'Disable clearAll if the list is attached to another control or if the list is empty.
            If Not IsNothing(Me.ExplorerCombo) Or Not IsNothing(Me.ExplorerTree) Then
                m_removeAll.Visible = False
                m_remove.Visible = False
                m_spacer1.Visible = False
            Else
                m_newFolder.Visible = False
            End If

            'Disable selectAll, RemoveAll and the sortMenu if the list is empty
            If Me.Items.Count < 1 Then
                m_selectAll.Enabled = False
                m_removeAll.Enabled = False
                sortMenu.Enabled = False
            End If

            Select Case Me.SelectedItems.Count
                Case 0
                    'Nothing to open
                    m_open.Visible = False
                    m_spacer1.Visible = False
                    'Nothing to remove
                    m_remove.Enabled = False
                Case 1
                    m_newFolder.Visible = False
                Case Is > 1
                    m_newFolder.Visible = False
                    'If more than one item is selected, check that none are folders
                    Dim i As Integer
                    For i = 0 To Me.SelectedItems.Count - 1
                        If Me.SelectedItems(i).CShItem.IsFolder Then
                            m_open.Enabled = False
                            Exit Select
                        End If
                    Next
            End Select
        End If
        If m_allowColumnChanges Then MyBase.ContextMenuStrip = columnContextMenu
    End Sub

    'Toggle view - Use Begin- and EndUpdate for flicker that occurs when the items are grouped
    Private Sub m_details_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_details.Click
        Me.BeginUpdate()
        Me.View = ExpListView.Details
        Me.EndUpdate()
    End Sub

    Private Sub m_list_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_list.Click
        Me.BeginUpdate()
        Me.View = ExpListView.List
        Me.EndUpdate()
    End Sub

    Private Sub m_icons_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_icons.Click
        Me.BeginUpdate()
        Me.View = ExpListView.LargeIcon
        Me.EndUpdate()
    End Sub

    Private Sub m_thumbnails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_thumbnails.Click
        Me.BeginUpdate()
        Me.View = ExpListView.Thumbnail
        Me.EndUpdate()
    End Sub

    Private Sub m_tiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_tiles.Click
        Me.BeginUpdate()
        Me.View = ExpListView.Tile
        Me.EndUpdate()
    End Sub

    'Select all list items
    Private Sub m_selectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_selectAll.Click
        Dim count As Integer
        For count = 0 To Me.Items.Count - 1
            Me.Items.Item(count).Selected = True
        Next
    End Sub

    'Opens a folder or runs/opens files
    Private Sub m_open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_open.Click
        OpenSelectedListItems()
    End Sub

    'Create a new folder
    Private Sub m_newfolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_newFolder.Click
        CreateNewFolder()
    End Sub

    'Refresh the list
    Private Sub m_refresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_refresh.Click
        RefreshList()
    End Sub

    'Empty list view
    Private Sub m_removeAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_removeAll.Click
        Me.Items.Clear()
    End Sub

    'Remove Selected Items
    Private Sub m_remove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_remove.Click
        Dim item As ExpListItem
        For Each item In Me.SelectedItems
            Me.Items.Remove(item)
        Next
    End Sub

    Private Sub m_showInGroups_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_showInGroups.Click
        If m_showInGroups.Checked Then
            ClearGroups()
            m_showInGroups.Checked = False
        Else
            GroupItems()
            m_showInGroups.Checked = True
        End If
    End Sub

#End Region

#Region "       New Folder "

    ''' <summary>
    ''' Creates a new folder in the list when the list is attached to an ExpCombo or ExpTree
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub CreateNewFolder()
        Dim path As String = ""
        If m_expCombo IsNot Nothing Then
            If m_expCombo.SelectedIndex < 0 Then Exit Sub
            path = m_expCombo.SelectedItem.CShItem.Path
        ElseIf m_expTree IsNot Nothing Then
            If m_expTree.SelectedItem Is Nothing Then Exit Sub
            path = m_expTree.SelectedItem.Path
        End If
        'Get new folder name
        Dim name As String = My.Resources.ControlStrings.NewFolder
        If Me.Columns(0).ColumnType <> ExpListColumnType.Name Or (Not m_allowRename) Then
            'Can't use label edit here
            name = InputBox(My.Resources.ControlStrings.EnterFolderName, My.Resources.ControlStrings.NewFolder, _
                , MousePosition.X, MousePosition.Y)
            If name.Trim = "" Then Exit Sub
        End If
        Dim n As Integer
        Dim newPath As String
        Dim tempPath As String = ""

        If path.Length = 3 Then
            newPath = path & name
        Else
            newPath = path & "\" & name
        End If

        Try
            If (Me.Columns(0).ColumnType = ExpListColumnType.Name) And (m_allowRename) Then
                For n = 1 To 10000
                    If IO.Directory.Exists(newPath & tempPath) Then
                        tempPath = "(" & n & ")"
                    Else
                        Exit For
                    End If
                    If n = 10000 Then
                        Throw New ApplicationException("Could not create a name for the new " & _
                         "folder: 'New Folder' and 'New Folder(1)' to 'New Folder(10000)' already exist!!!")
                    End If
                Next
                newPath = newPath & tempPath
            End If
            If Not IO.Directory.Exists(newPath) Then
                IO.Directory.CreateDirectory(newPath)
                RefreshList()
                'Select newly added folder
                Me.Select()
                Dim itm As ExpListItem
                For Each itm In Me.Items
                    If itm.CShItem.Path = newPath Then
                        'Post the Event
                        OnNewFolder(New ExpListItemEventArgs(itm))
                        'Select the item and start editing... or not
                        itm.Selected = True
                        If m_allowRename Then
                            itm.BeginEdit()
                        Else
                            itm.EnsureVisible()
                        End If
                        Exit For
                    End If
                Next
            Else
                MessageBox.Show(My.Resources.ControlStrings.CouldNotCreateFolder & vbCrLf & newPath, _
                         My.Resources.ControlStrings._Error, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, _
                         My.Resources.ControlStrings._Error, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

#End Region

#Region "       Grouping "

    ''' <summary>
    ''' Groups the items reletive to the current sorting column
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub GroupItems()
        'Exit if the control isn't sorted
        If m_sortOrder = SortOrder.None Then Exit Sub

        Dim column As ExpListColumn = m_sortColumn

        'Store the current cursor and show the hourglass
        Dim oldCursor As Cursor = Me.Parent.Cursor
        Dim hasParent As Boolean = Not IsNothing(Me.Parent)
        If hasParent Then Me.Parent.Cursor = Cursors.WaitCursor
        Me.BeginUpdate()

        'Raise the BeforeGroup Event to enable custom grouping
        Dim args As New ExpListGroupEventArgs(column)
        OnGroup(args)

        'Set the groups
        SetItemGroups(column)

        m_showInGroups.Checked = True

        'Reset the cursor
        Me.EndUpdate()
        If hasParent Then Me.Parent.Cursor = oldCursor
    End Sub

    ''' <summary>
    ''' Clears any groups in the list
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overridable Sub ClearGroups()
        Me.Groups.Clear()
        Me.Sort()
        m_showInGroups.Checked = False
        Me.Refresh()
    End Sub


    Private Sub SetItemGroups(ByVal column As ExpListColumn)

        'If we are using custom grouping then exit
        If column.CustomGrouping Then Exit Sub

        Dim item As ExpListItem
        Dim groupName As String = ""
        For Each item In Me.Items
            Dim shItem As CShItem = item.CShItem
            Select Case column.ColumnType
                Case ExpListColumnType.Name
                    'Use the first letter of the display name
                    groupName = shItem.DisplayName.Substring(0, 1).ToUpper

                Case ExpListColumnType.Path
                    'Use the path without the display name
                    Dim path As String = shItem.Path
                    Dim index As Integer = path.LastIndexOf("\"c)
                    If index = -1 Then
                        'Check for a web file/folder
                        index = path.LastIndexOf("/"c)
                    End If
                    If index = -1 Or path.StartsWith("::") Then
                        groupName = ""
                    Else
                        groupName = path.Substring(0, index + 1)
                    End If

                Case ExpListColumnType.Created, ExpListColumnType.Modified, ExpListColumnType.CustomDate
                    If item.SubItems(column.Index).Text = "" Then
                        groupName = My.Resources.ControlStrings.Unspecified
                    Else
                        Try
                            groupName = GetDateGroup(CDate(item.SubItems(column.Index).Text))
                        Catch ex As Exception
                            'Just return "Unspecified"
                            groupName = My.Resources.ControlStrings.Unspecified
                        End Try
                    End If

                Case ExpListColumnType.Type
                    With shItem
                        'Some special strings are defined here
                        'You could probably get them from the system
                        'but to be honest, I couldn't be bothered.
                        If .IsRemovable Then
                            groupName = My.Resources.ControlStrings.Removable
                        ElseIf .IsNetworkDrive Then
                            groupName = My.Resources.ControlStrings.NetworkShare
                        ElseIf .IsDisk Then
                            groupName = My.Resources.ControlStrings.LocalDisk
                        ElseIf Me.isMyComputer Then
                            If .IsFolder And Not .TypeName = CShItem.strSystemFolder And _
                                Me.isMyComputer Then
                                groupName = My.Resources.ControlStrings.FilesStoredOnThisComputer
                            Else
                                groupName = My.Resources.ControlStrings.Other
                            End If
                        Else
                            groupName = .TypeName
                        End If
                    End With

                Case ExpListColumnType.Size
                    'A few more strings for the resx file - ugh.
                    Dim size As Long = shItem.Length
                    Dim MB As Long = 1024 * 1024
                    Dim GB As Long = 1024 * 1024 * 1024
                    If shItem.IsDisk Then 'For Disks
                        If size >= (60 * GB) Then
                            groupName = My.Resources.ControlStrings.Gigantic
                        ElseIf size >= (25 * GB) Then
                            groupName = My.Resources.ControlStrings.Huge
                        ElseIf size >= (10 * GB) Then
                            groupName = My.Resources.ControlStrings.Large
                        ElseIf size >= (2 * GB) Then
                            groupName = My.Resources.ControlStrings.Medium
                        ElseIf size >= (500 * MB) Then
                            groupName = My.Resources.ControlStrings.Small
                        ElseIf size > 0 Then
                            groupName = My.Resources.ControlStrings.Tiny
                        Else
                            'Only a CD-ROM without a disk will have 0 bytes...
                            groupName = My.Resources.ControlStrings.Unspecified
                        End If
                    ElseIf shItem.IsFolder And Not Me.isMyComputer Then
                        'MyComputer shows folders as Unspecified...
                        groupName = My.Resources.ControlStrings.Folder
                    ElseIf (Not shItem.IsFolder And shItem.IsFileSystem) Then 'For files and folders
                        If size >= GB Then
                            groupName = My.Resources.ControlStrings.Gigantic
                        ElseIf size >= (15 * MB) Then
                            groupName = My.Resources.ControlStrings.Huge
                        ElseIf size >= MB Then
                            groupName = My.Resources.ControlStrings.Large
                        ElseIf size >= (100 * 1024) Then
                            groupName = My.Resources.ControlStrings.Medium
                        ElseIf size >= (32 * 1024) Then
                            groupName = My.Resources.ControlStrings.Small
                        ElseIf size > 0 Then
                            groupName = My.Resources.ControlStrings.Tiny
                        Else
                            groupName = My.Resources.ControlStrings.Zero
                        End If
                    Else
                        groupName = My.Resources.ControlStrings.Unspecified
                    End If

                Case ExpListColumnType.CustomText, ExpListColumnType.CustomNumeric
                    groupName = item.SubItems(column.Index).Text

            End Select

            If IsNothing(Me.Groups(groupName)) Then
                item.Group = Me.Groups.Add(groupName, groupName)
            Else
                item.Group = Me.Groups(groupName)
            End If
        Next
    End Sub

    Private Function GetDateGroup(ByVal dateToProcess As Date) As String
        'This is all pretty self-explanatory.
        Dim todaysDate As Date = Date.Today

        If Date.Today.ToShortDateString = dateToProcess.ToShortDateString Then
            Return My.Resources.ControlStrings.Today
        ElseIf dateToProcess.AddDays(1).ToShortDateString = todaysDate.ToShortDateString Then
            Return My.Resources.ControlStrings.Yesterday
        ElseIf dateToProcess.ToShortDateString = todaysDate.AddDays(-2).ToShortDateString Then
            Return My.Resources.ControlStrings.TwoDaysAgo
        ElseIf dateToProcess.AddDays(Date.Today.DayOfWeek) >= todaysDate Then
            Return My.Resources.ControlStrings.EarlierThisWeek
        ElseIf dateToProcess.AddDays(Date.Today.DayOfWeek + 7) >= todaysDate Then
            Return My.Resources.ControlStrings.LastWeek
        ElseIf dateToProcess.AddDays(Date.Today.DayOfWeek + 14) >= todaysDate Then
            Return My.Resources.ControlStrings.TwoWeeksAgo
        ElseIf dateToProcess.Year = todaysDate.Year Then
            If dateToProcess.Month = todaysDate.Month Then
                Return My.Resources.ControlStrings.EarlierThisMonth
            ElseIf dateToProcess.AddMonths(1).Month = todaysDate.Month Then
                Return My.Resources.ControlStrings.LastMonth
            ElseIf dateToProcess.AddMonths(2).Month = todaysDate.Month Then
                Return My.Resources.ControlStrings.TwoMonthsAgo
            Else
                Return My.Resources.ControlStrings.EarlierThisYear
            End If
        ElseIf dateToProcess.AddYears(1).Year = todaysDate.Year Then
            Return My.Resources.ControlStrings.LastYear
        ElseIf dateToProcess.AddYears(2).Year = todaysDate.Year Then
            Return My.Resources.ControlStrings.TwoYearsAgo
        Else
            Return My.Resources.ControlStrings.ALongTimeAgo
        End If
    End Function

#End Region

#Region "       Thumbnails "

    ''' <summary>
    ''' Draws ExpListItems when View is set to Thumbnails
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnDrawItem(ByVal e As System.Windows.Forms.DrawListViewItemEventArgs)
        'There are a lot of hard-coded sizes here - it was the easiest way and seems to work...
        Dim itm As ExpListItem = DirectCast(e.Item, ExpListItem)
        Dim img As Image = itm.Thumbnail
        Dim offset As Integer = CInt((e.Bounds.Width - 95) / 2)

        'Fill the background
        e.Graphics.FillRectangle(New SolidBrush(itm.BackColor), _
                New Rectangle(e.Bounds.X, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height))

        'Draw the image - or the XL icon
        If img IsNot Nothing Then
            e.Graphics.DrawImage(img, _
                    New Point(e.Bounds.X + offset, e.Bounds.Y + offset - 5))
        Else
            Dim ic As Icon = SystemImageListManager.GetXLIcon(itm.ImageIndex)
            Dim x As Integer = CInt(e.Bounds.X + offset + ((96 - ic.Width) / 2))
            Dim y As Integer = CInt(e.Bounds.Y + offset + ((86 - ic.Height) / 2))
            e.Graphics.DrawIcon(ic, x, y)
            ShellDll.DestroyIcon(ic.Handle)
            If itm.CShItem.IsLink Then
                'Get the overlay icon and draw it in the bottom corner
                'Hopefully the icon is always at index 1... couldn't find
                'a way to get it properly...
                ic = SystemImageListManager.GetIcon(1)
                e.Graphics.DrawIcon(ic, e.Bounds.X + offset, y + 40)
                ShellDll.DestroyIcon(ic.Handle)
            End If
        End If

        'Draw the item text
        Dim format As StringFormat = GetFormat()
        If itm.Selected Then
            Dim size As SizeF = e.Graphics.MeasureString(itm.Text, Me.Font, _
                                New SizeF(CInt(e.Bounds.Width), CInt(e.Bounds.Height) - 112), format)
            e.Graphics.DrawRectangle(New Pen(System.Drawing.SystemColors.Highlight, 3), _
                                New Rectangle(e.Bounds.X + offset - 2, e.Bounds.Y + offset - 7, 99, 99))
            e.Graphics.FillRectangle(New SolidBrush(System.Drawing.SystemColors.Highlight), New Rectangle( _
                                e.Bounds.X + CInt((e.Bounds.Width - size.Width) / 2), e.Bounds.Y + 111, _
                                CInt(size.Width), CInt(size.Height)))
            e.Graphics.DrawString(itm.Text, Me.Font, SystemBrushes.HighlightText, _
                                New Rectangle(e.Bounds.X, e.Bounds.Y + 111, _
                                CInt(e.Bounds.Width), CInt(e.Bounds.Height) - 112), format)

        Else
            e.Graphics.DrawRectangle(Pens.LightGray, New Rectangle(e.Bounds.X + offset, _
                                e.Bounds.Y + offset - 5, 95, 95))
            Dim size As SizeF = e.Graphics.MeasureString(itm.Text, Me.Font, _
                                New SizeF(CInt(e.Bounds.Width), CInt(e.Bounds.Height)), format)
            e.Graphics.DrawString(itm.Text, Me.Font, New SolidBrush(Me.ForeColor), _
                                New Rectangle(e.Bounds.X, e.Bounds.Y + 111, _
                                CInt(e.Bounds.Width), CInt(e.Bounds.Height) - 112), format)

            'This only works for a single line and doesn't allow me to change text colour :(
            'e.DrawText(TextFormatFlags.Bottom Or TextFormatFlags.HorizontalCenter Or _
            '     TextFormatFlags.EndEllipsis Or TextFormatFlags.NoPadding Or TextFormatFlags.GlyphOverhangPadding)
        End If
    End Sub

#End Region

#Region "       Visible Changed "

    Private Sub ExpList_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.VisibleChanged
        If Me.Visible Then
            SetListViewBackgroundImage(m_backgroundimage, m_backgroundImageAlignment)
            'Required for dialogs (on the second call to ShowDialog the system image list has been detached)
            SystemImageListManager.SetListViewImageList(Me, True, False)
            SystemImageListManager.SetListViewImageList(Me, False, False)
            'Required after a change in .Net 2.0 where this call only works after the ExpList has
            'been added to the parent form
            SendMessage(Me.Handle, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_LABELTIP, LVS_EX_LABELTIP)
            'To Fix a bug where the system image list is used for the column headers on startup
            SetSortIcon()
        End If
    End Sub

#End Region

#Region "       BackgroundImage "

    ''' <summary>
    ''' Sets the ExpList background image
    ''' </summary>
    ''' <param name="backImage">The path to the image file to be used as the background</param>
    ''' <param name="alignment">The alignment to be used for the background image</param>
    ''' <remarks></remarks>
    Protected Overridable Sub SetListViewBackgroundImage(ByVal backImage As String, ByVal alignment As ContentAlignment)
        'Set Alignment
        Dim xOffset As Integer
        Dim yOffset As Integer
        Select Case alignment
            Case ContentAlignment.BottomCenter
                xOffset = 50
                yOffset = 100
            Case ContentAlignment.BottomLeft
                xOffset = 0
                yOffset = 100
            Case ContentAlignment.BottomRight
                xOffset = 100
                yOffset = 100
            Case ContentAlignment.MiddleCenter
                xOffset = 50
                yOffset = 50
            Case ContentAlignment.MiddleLeft
                xOffset = 0
                yOffset = 50
            Case ContentAlignment.MiddleRight
                xOffset = 100
                yOffset = 50
            Case ContentAlignment.TopCenter
                xOffset = 50
                yOffset = 0
            Case ContentAlignment.TopLeft
                xOffset = 0
                yOffset = 0
            Case ContentAlignment.TopRight
                xOffset = 100
                yOffset = 0
        End Select
        'Set background image.
        Dim lbi As New LVBKIMAGE
        With lbi
            If backImage = "" Then
                .ulFlags = LVBKIF_SOURCE_NONE
            Else
                .pszImage = backImage & Convert.ToChar(0)
                .cchImageMax = Len(.pszImage)
                .ulFlags = LVBKIF_SOURCE_URL Or LVBKIF_STYLE_NORMAL
                .xOffsetPercent = xOffset
                .yOffsetPercent = yOffset
            End If
        End With
        SendMessage(Me.Handle, LVM_SETTEXTBKCOLOR, 0, CLR_NONE)
        SendMessage(Me.Handle, LVM_SETBKIMAGE, 0, lbi)
    End Sub

#End Region

#Region "       Rename "

    ''' <summary>
    ''' AfterLabelEdit Event Handler fot the ExpList
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub AfterRename(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles MyBase.AfterLabelEdit
        Dim itm As ExpListItem
        Dim newName As String
        Try
            newName = e.Label.ToString
        Catch ex As Exception
            e.CancelEdit = True
            Exit Sub
        End Try
        itm = Me.Items(e.Item)
        RenameItem(itm, NewName)
    End Sub

    ''' <summary>
    ''' BeforeLabelEdit Event Handler fot the ExpList
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub BeforeRename(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles MyBase.BeforeLabelEdit
        'CancelEdit if the file name is not the first column
        If Me.Columns(0).ColumnType <> ExpListColumnType.Name Or (Not m_allowRename) Then
            e.CancelEdit = True
        End If
        'CancelEdit if the item cannot be edited - is not in the file system
        Dim item As CShItem = Me.Items(e.Item).CShItem
        If (Not item.IsFileSystem) Or item.IsDisk Or item.IsSharePoint Then
            e.CancelEdit = True
        End If

        If Me.View = ExpListView.Thumbnail Then
            'Reset the handled variable for the TextBox.Leave event
            handled = False
            'Cancel the edit - we'll handle this ourselves
            e.CancelEdit = True
            'Set up the TextBox
            If Not Me.Controls.Contains(editTextBox) Then
                Me.Controls.Add(editTextBox)
            End If
            editTextBox.BringToFront()
            editTextBox.Visible = True
            editTextBox.Focus()
            editTextBox.Tag = Me.Items(e.Item)
            editTextBox.Text = Me.Items(e.Item).CShItem.DisplayName
            editTextBox.SelectAll()
            UpdateTextBox()
            'Remove the text from the ExpListItem in the background
            Me.Items(e.Item).Text = ""
        End If
    End Sub

    Private Sub editTextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles editTextBox.TextChanged
        UpdateTextBox()
    End Sub

    Private Sub UpdateTextBox()
        If IsNothing(editTextBox.Tag) Then Exit Sub

        Dim itm As ExpListItem = DirectCast(editTextBox.Tag, ExpListItem)

        Dim format As StringFormat = GetFormat()
        Dim g As Graphics = Me.CreateGraphics
        Dim size As SizeF = g.MeasureString(editTextBox.Text, editTextBox.Font, _
                            CInt(itm.Bounds.Width), format)
        'Check that the TextBox does not disappear out the end of the list
        If itm.Bounds.Y + 110 > Me.ClientSize.Height - CInt(size.Height) - 5 Then
            editTextBox.Height = Me.ClientSize.Height - editTextBox.Top - 5
        Else
            editTextBox.Height = CInt(size.Height) + 5
        End If
        editTextBox.Width = CInt(size.Width) + 5
        editTextBox.Location = New Point((itm.Bounds.X + CInt((itm.Bounds.Width - size.Width) / 2)), _
                                                (itm.Bounds.Y + 110))
    End Sub

    Private Sub editTextBox_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles editTextBox.KeyPress
        If e.KeyChar = Chr(13) Then
            Dim itm As ExpListItem = DirectCast(editTextBox.Tag, ExpListItem)
            RenameItem(itm, editTextBox.Text)
            'Hide the TextBox
            editTextBox.Visible = False
            OnAfterLabelEdit(New LabelEditEventArgs(itm.Index))
            e.Handled = True
            'Set the handled variable for the TextBox.Leave event
            handled = True
        End If
    End Sub

    Private Sub editTextBox_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles editTextBox.Leave
        'If handled is true then return has been pressed and we have already handled the event
        If Not handled And editTextBox.Visible Then
            Dim itm As ExpListItem = DirectCast(editTextBox.Tag, ExpListItem)
            RenameItem(itm, editTextBox.Text)
            'Hide the TextBox
            editTextBox.Visible = False
            OnAfterLabelEdit(New LabelEditEventArgs(itm.Index))
        End If
    End Sub

    Private Sub RenameItem(ByVal itm As ExpListItem, ByVal newName As String)

        If Not itm.CShItem.DisplayName.ToLower = newName.ToLower Then

            Dim oldName As String
            Dim newPath As String
            Dim index As Integer
            Dim path As String
            path = itm.CShItem.Path

            oldName = itm.CShItem.GetFileName
            index = path.LastIndexOf("\"c)
            newPath = path.Remove(index + 1, oldName.Length)
            Try
                If itm.CShItem.IsFolder Then
                    newPath &= newName
                    IO.Directory.Move(path.ToString, newPath)
                ElseIf itm.CShItem.IsFileSystem Then
                    newName = oldName.Replace(itm.CShItem.DisplayName, newName)
                    newPath &= newName
                    IO.File.Move(path.ToString, newPath)
                End If
                'Refresh the CShItem associated with this item - this gets a bit silly
                'for My Documents and sub-folders, if we just generate a new CShItem then the full path
                'is used instead of the current Parent (e.g. directly under the desktop)
                If itm.CShItem.Path.StartsWith(CShItem.GetCShItem(ShellDll.CSIDL.MYDOCUMENTS).Path) Then
                    Dim shItm As CShItem
                    Dim shItms As New ArrayList
                    If itm.CShItem.IsFolder Then
                        shItms = itm.CShItem.Parent.GetDirectories
                    ElseIf itm.CShItem.IsFileSystem Then
                        shItms = itm.CShItem.Parent.GetFiles
                    End If
                    For Each shItm In shItms
                        If shItm.Path = newPath Then
                            itm.CShItem = shItm
                            Exit For
                        End If
                    Next
                Else
                    'Can't use this if My Documents is the parent folder
                    itm.CShItem = CShItem.GetCShItem(newPath)
                End If
            Catch ex As Exception
                MessageBox.Show(ex.Message, _
                             My.Resources.ControlStrings._Error, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If
        itm.Text = itm.CShItem.DisplayName
        itm.ToolTipText = itm.CShItem.DisplayName & ControlChars.CrLf _
                    & itm.CShItem.TypeName & ControlChars.CrLf & itm.CShItem.Size
    End Sub

    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
        MyBase.WndProc(m)
        If IsNothing(editTextBox) Then Exit Sub
        If m.Msg = &HF And editTextBox.Visible Then '&HF = WM_PAINT
            UpdateTextBox()
        End If
    End Sub
    Private Function GetFormat() As StringFormat
        Dim format As New StringFormat()
        format.Alignment = StringAlignment.Center
        format.FormatFlags = StringFormatFlags.LineLimit
        format.Trimming = StringTrimming.EllipsisCharacter
        Return format
    End Function

#End Region

#Region "       Protected Event Raising Methods "

    ''' <summary>
    ''' Raises the FileActivate Event when an ExpListItem that represents a file is activated
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnFileActivate(ByVal e As ExpListItemEventArgs)
        RaiseEvent FileActivate(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the FolderActivate Event when an ExpListItem that represents a folder is activated
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnFolderActivate(ByVal e As ExpListItemEventArgs)
        RaiseEvent FolderActivate(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the BeforeSort Event directly before the ExpList is sorted
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnBeforeSort(ByVal e As ExpListSortEventArgs)
        RaiseEvent BeforeSort(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the AfterSort Event directly after the ExpList is sorted
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnAfterSort(ByVal e As ExpListSortEventArgs)
        RaiseEvent AfterSort(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the Group Event directly before the ExpList is grouped
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnGroup(ByVal e As ExpListGroupEventArgs)
        RaiseEvent Group(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the ItemsAdded Event directly after new Items have been added to the ExpList
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Friend Overridable Sub OnItemsAdded(ByVal e As ExpListItemsAddedEventArgs)
        RaiseEvent ItemsAdded(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the SelectedFolderChanged Event.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks> If we don't post this Event the Browser (or any other Parent) will handle the Selected[Index/Node]Changed 
    ''' Event of an attached ExpCombo/ExpTree before the ExpList - i.e. before the Items have been added to the list.</remarks>
    Protected Overridable Sub OnSelectedFolderChanged(ByVal e As EventArgs)
        RaiseEvent SelectedFolderChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the NewFolder Event when a new folder is created in the ExpList
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnNewFolder(ByVal e As ExpListItemEventArgs)
        RaiseEvent NewFolder(Me, e)
    End Sub

#End Region

#End Region

End Class

