Imports System.Runtime.InteropServices
Partial Class ExpList

#Region " ExpListItemCollection "

    ''' <summary>
    ''' Collection Class for ExpListItems
    ''' </summary>
    ''' <remarks></remarks>
    <ClassInterface(ClassInterfaceType.None), Guid("34C5285B-E8E9-4337-98BC-5A9417F0C8B1")> _
    Public Class ExpListItemCollection
        Inherits ListView.ListViewItemCollection

        Friend Sub New(ByVal owner As ExpList)
            MyBase.New(owner)
            m_expList = owner
        End Sub

        ''' <summary>
        ''' Returns an ExpListItem at the specified index in the collection
        ''' </summary>
        ''' <param name="displayIndex">The index of the Item to be returned</param>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Default Public Overridable Shadows ReadOnly Property Item(ByVal displayIndex As Integer) As ExpListItem
            Get
                Return DirectCast(MyBase.Item(displayIndex), ExpListItem)
            End Get
        End Property

        Private m_expList As ExpList
        ''' <summary>
        ''' The parent ExpList object
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property ExpList() As ExpList
            Get
                Return m_expList
            End Get
        End Property

        ''' <summary>
        ''' Adds an ExpListItem to the end of the collection.
        ''' </summary>
        ''' <param name="itm">The CShItem to be associated with the new ExpListItem</param>
        ''' <returns>The added ExpListItem. Will return Nothing if the item could not be added</returns>
        ''' <remarks></remarks>
        Public Overridable Shadows Function Add(ByVal itm As CShItem) As ExpListItem
            Dim listItem As New ExpListItem(itm)
            Return Add(listItem)
        End Function

        ''' <summary>
        ''' Adds an ExpListItem to the end of the collection.
        ''' </summary>
        ''' <param name="path">The path for the new ExpListItem</param>
        ''' <returns>The added ExpListItem. Will return Nothing if the item could not be added</returns>
        ''' <remarks></remarks>
        Public Overridable Shadows Function Add(ByVal path As String) As ExpListItem
            'Gets ShellItem from path and adds it to ListView
            Return Add(New ExpListItem(CShItem.GetCShItem(path)))
        End Function

        ''' <summary>
        ''' Adds an ExpListItem to the end of the collection.
        ''' </summary>
        ''' <param name="value">The ExpListItem to add</param>
        ''' <returns>The added ExpListItem. Will return Nothing if the item could not be added</returns>
        ''' <remarks></remarks>
        Public Overridable Shadows Function Add(ByVal value As ExpListItem) As ExpListItem

            'Make sure the item has been initialized
            If IsNothing(value.CShItem) Then
                Throw New ApplicationException("The CShItem property must be set before adding the item to the collection.")
            End If

            'Check to see if the item is hidden
            If value.CShItem.IsHidden And Not m_expList.ShowHiddenFiles Then
                Return Nothing
            End If
            'Set the cursor to the hourglass
            m_expList.UseWaitCursor = True
            m_expList.BeginUpdate()

            'Filters files by extension
            If Not IsNothing(m_expList.Filter) Then
                If Not m_expList.FilterItem(value.CShItem) Then
                    m_expList.UseWaitCursor = False
                    m_expList.EndUpdate()
                    Return Nothing
                End If
            End If

            'Clear any groups
            Dim grouped As Boolean = m_expList.Grouped
            If grouped Then m_expList.Grouped = False

            m_expList.SetItemText(value)
            MyBase.Add(value)

            'Set the ItemState to Cut for Hidden files/folders
            If value.CShItem.IsHidden Then value.State = ExpListItemState.Cut

            'Regroup the new Items
            If grouped Then
                m_expList.Grouped = True
            End If

            'Reset the cursor
            m_expList.UseWaitCursor = False

            'Raise the ItemsAdded Event
            Dim itemList(0) As ExpListItem
            itemList(0) = value
            m_expList.OnItemsAdded(New ExpListItemsAddedEventArgs(itemList))
            m_expList.EndUpdate()
            Return value
        End Function

        ''' <summary>
        ''' Adds a range of ExpListItems to the collection
        ''' </summary>
        ''' <param name="items">An Array of ExpListItems to be added</param>
        ''' <remarks></remarks>
        Public Overridable Shadows Sub AddRange(ByVal items() As ExpListItem)
            Dim itm As ExpListItem
            For Each itm In items
                Add(itm)
            Next
        End Sub

        ''' <summary>
        ''' Adds a range of CShItems to the collection
        ''' </summary>
        ''' <param name="items">An Array of CShItems to be added</param>
        ''' <returns>An Array of the ExpListItems that were actually added</returns>
        ''' <remarks></remarks>
        Public Overridable Shadows Function AddRange(ByVal items() As CShItem) As ExpListItem()
            'Filter the items
            items = FilterListItems(items)
            'Declare a new ExpListItemCollection for the EventArgs
            Dim itemsList As New ArrayList


            'Set the cursor to the hourglass
            m_expList.UseWaitCursor = True
            m_expList.BeginUpdate()

            'Clear any groups
            Dim grouped As Boolean = m_expList.Grouped
            If grouped Then m_expList.Grouped = False

            Dim itm As CShItem
            For Each itm In items
                'Check to see if the item is hidden
                If itm.IsHidden And Not m_expList.ShowHiddenFiles Then GoTo NEXTITEM

                'Adds list item
                Dim listItem As New ExpListItem(itm)
                m_expList.SetItemText(listItem)
                MyBase.Add(listItem)
                itemsList.Add(listItem)
                'Set the ItemState to Cut for Hidden files/folders
                If itm.IsHidden Then listItem.State = ExpListItemState.Cut
NEXTITEM:
            Next

            'Regroup the new Items
            If grouped Then
                m_expList.Grouped = True
            End If

            'Reset the cursor
            m_expList.UseWaitCursor = False

            'Raise the ItemsAdded Event
            Dim itms() As ExpListItem = CType(itemsList.ToArray(GetType(ExpListItem)), ExpListItem())
            m_expList.OnItemsAdded(New ExpListItemsAddedEventArgs(itms))
            m_expList.EndUpdate()
            Return itms
        End Function

        ''' <summary>
        ''' Inserts a new CShItem at the specified index in the collection
        ''' </summary>
        ''' <param name="index">The index at which to insert the CShItem</param>
        ''' <param name="itm">The CShItem to add</param>
        ''' <remarks></remarks>
        Public Shadows Sub Insert(ByVal index As Integer, ByVal itm As CShItem)
            Insert(index, New ExpListItem(itm))
        End Sub

        ''' <summary>
        ''' Inserts a new ExpListItem at the specified index in the collection
        ''' </summary>
        ''' <param name="index">The index at which to insert the ExpListItem</param>
        ''' <param name="path">The path for the new ExpListItem</param>
        ''' <remarks></remarks>
        Public Shadows Sub Insert(ByVal index As Integer, ByVal path As String)
            Insert(index, New ExpListItem(CShItem.GetCShItem(path)))
        End Sub

        ''' <summary>
        ''' Inserts a new ExpListItem at the specified index in the collection
        ''' </summary>
        ''' <param name="index">The index at which to insert the ExpListItem</param>
        ''' <param name="value">The ExpListItem to add</param>
        ''' <remarks></remarks>
        Public Shadows Sub Insert(ByVal index As Integer, ByVal value As ExpListItem)

            'Make sure the item has been initialized
            If IsNothing(value.CShItem) Then
                Throw New ApplicationException("The CShItem property must be set before adding the item to the collection.")
            End If

            If value.CShItem.IsHidden And Not m_expList.ShowHiddenFiles Then
                Exit Sub
            End If

            'Set the cursor to the hourglass
            m_expList.UseWaitCursor = True
            m_expList.BeginUpdate()

            'Filters files by extension
            If Not IsNothing(m_expList.Filter) Then
                If Not m_expList.FilterItem(value.CShItem) Then
                    m_expList.UseWaitCursor = False
                    m_expList.EndUpdate()
                    Exit Sub
                End If
            End If

            MyBase.Insert(index, value)
            m_expList.SetItemText(value)

            'Set the ItemState to Cut for Hidden files/folders
            If value.CShItem.IsHidden Then value.State = ExpListItemState.Cut

            'Reset the cursor
            m_expList.UseWaitCursor = False

            'Raise the ItemsAdded Event
            Dim itemList(0) As ExpListItem
            itemList(0) = value
            m_expList.OnItemsAdded(New ExpListItemsAddedEventArgs(itemList))
            m_expList.EndUpdate()
        End Sub

        'Filter for multiple files - return an Array of CShItems
        Private Function FilterListItems(ByVal items() As CShItem) As CShItem()
            'Returns true if file extension is in the m_filter array
            Dim results As New ArrayList
            If IsNothing(m_expList.Filter) Then
                Return items
            End If
            Dim item As CShItem
            For Each item In items
                If item.IsFolder Then
                    'Add all folders to the results list
                    results.Add(item)
                Else
                    Dim count As Integer
                    For count = 0 To m_expList.Filter.Length - 1
                        'Check the extension against the characters after the point in the filter string
                        If item.GetFileName.ToLower.Substring(item.GetFileName.LastIndexOf("."c) + 1) = _
                            m_expList.Filter(count).ToLower.Substring(m_expList.Filter(count).LastIndexOf("."c) + 1).Trim Then
                            results.Add(item)
                        End If
                    Next
                End If
            Next
            Return CType(results.ToArray(GetType(CShItem)), CShItem())
        End Function

    End Class

#End Region

#Region " SelectedExpListItemCollection "

    ''' <summary>
    ''' Collection Class for currently selected ExpListItems
    ''' </summary>
    ''' <remarks></remarks>
    <ClassInterface(ClassInterfaceType.None), Guid("5801F13A-7A11-4263-8A91-B474BF92679E")> _
    Public Class SelectedExpListItemCollection
        Inherits ListView.SelectedListViewItemCollection

        Friend Sub New(ByVal owner As ExpList)
            MyBase.New(owner)
            m_expList = owner
        End Sub

        Private m_expList As ExpList
        ''' <summary>
        ''' The parent ExpList object
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property ExpList() As ExpList
            Get
                Return m_expList
            End Get
        End Property

        ''' <summary>
        ''' Returns an ExpListItem at the specified index in the collection
        ''' </summary>
        ''' <param name="displayIndex">The index of the Item to be returned</param>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Default Public Overridable Shadows ReadOnly Property Item(ByVal displayIndex As Integer) As ExpListItem
            Get
                Return DirectCast(MyBase.Item(displayIndex), ExpListItem)
            End Get
        End Property

    End Class

#End Region

#Region " ExpListColumnCollection "

    ''' <summary>
    ''' Collection Class for the columns in the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    <ClassInterface(ClassInterfaceType.None), Guid("20A63C50-72C6-417d-8158-C78AC2265FC4")> _
    Public Class ExpListColumnCollection
        Inherits ListView.ColumnHeaderCollection

        Private m_expList As ExpList

        Friend Sub New(ByVal owner As ExpList)
            MyBase.New(owner)
            m_expList = owner
        End Sub

        ''' <summary>
        ''' The parent ExpList object
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property ExpList() As ExpList
            Get
                Return m_expList
            End Get
        End Property

        ''' <summary>
        ''' Returns an ExpListItem at the specified index in the collection
        ''' </summary>
        ''' <param name="index">The index of the Item to be returned</param>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Default Public Overridable Shadows ReadOnly Property Item(ByVal index As Integer) As ExpListColumn
            Get
                Return DirectCast(MyBase.Item(index), ExpListColumn)
            End Get
        End Property


        ''' <summary>
        ''' Inserts an ExpListColumn into the collection
        ''' </summary>
        ''' <param name="index">The index at which to insert the ExpListColumn</param>
        ''' <param name="value">The ExpListColumn to be inserted</param>
        ''' <remarks></remarks>
        Public Shadows Sub Insert(ByVal index As Integer, ByVal value As ExpListColumn)
            'Check if the column has already been added
            'This is required for designer support
            If m_expList.Columns.IndexOf(value) > -1 Then Exit Sub

            'Add column first - otherwise the column.Index property will
            'not be initialized for the SetUpColumn method
            MyBase.Insert(index, value)
            SetUpColumn(value)
        End Sub

        ''' <summary>
        ''' Adds an ExpListColumn to the end of the collection
        ''' </summary>
        ''' <param name="value">The ExpListColumn to be added</param>
        ''' <returns>The index of the inserted ExpListColumn</returns>
        ''' <remarks></remarks>
        Public Overridable Shadows Function Add(ByVal value As ExpListColumn) As Integer
            'Check if the column has already been added
            'This is required for designer support
            If m_expList.Columns.IndexOf(value) > -1 Then
                Add = m_expList.Columns.IndexOf(value)
            Else
                Add = MyBase.Add(value)
            End If

            'Add column first - otherwise the column.Index property will
            'not be initialized for the SetUpColumn method
            SetUpColumn(value)
        End Function

        ''' <summary>
        ''' Adds an Array of ExpListColumns to the collection
        ''' </summary>
        ''' <param name="values">The Array of ExpListColumns to be added</param>
        ''' <remarks></remarks>
        Public Overridable Shadows Sub AddRange(ByVal values() As ExpListColumn)
            Dim value As ExpListColumn
            For Each value In values
                Me.Add(value)
            Next
        End Sub

        ''' <summary>
        ''' Removes the specified ExpListColumn from the collection
        ''' </summary>
        ''' <param name="column">The ExpListColumn to be removed</param>
        ''' <remarks></remarks>
        Public Overridable Shadows Sub Remove(ByVal column As ExpListColumn)
            'Check that the column exists in the collection
            If m_expList.Columns.IndexOf(column) < 0 Then Exit Sub

            'Column Text is removed when the base class calls RemoveAt
            MyBase.Remove(column)
            m_expList.sortMenu.DropDownItems.Remove(column.sortMenuItem)
            column.contextMenuItem.Checked = False
        End Sub

        ''' <summary>
        ''' Removes the ExpListColumn at the specified index in the collection
        ''' </summary>
        ''' <param name="index">The index of the ExpListColumn to be removed</param>
        ''' <remarks></remarks>
        Public Overrides Sub RemoveAt(ByVal index As Integer)
            'Check that the column exists in the collection
            If m_expList.Columns.Count <= index Then Exit Sub


            m_expList.sortMenu.DropDownItems.Remove(Me.Item(index).sortMenuItem)
            Me.Item(index).contextMenuItem.Checked = False
            'TODO: Bug - After RemoveAt the widths are reset!?!?!
            Dim column As ExpListColumn = Me.Item(index)
            Dim width As Integer = column.Width
            MyBase.RemoveAt(index)
            column.Width = width

            'Remove the column text - must be done after the column has been removed
            Dim item As ExpListItem
            For Each item In m_expList.Items
                item.SubItems.RemoveAt(index)
            Next
        End Sub

        ''' <summary>
        ''' Clears the collection
        ''' </summary>
        ''' <remarks></remarks>
        Public Overrides Sub Clear()
            MyBase.Clear()
            Dim column As ExpListColumn
            Dim item As ExpListItem
            For Each column In Me
                'Clear the Item Text
                For Each item In m_expList.Items
                    item.SubItems(Me.IndexOf(column)).Text = ""
                Next
                'Remove all the Columns and their Menu Items
                m_expList.sortMenu.DropDownItems.Remove(column.sortMenuItem)
                m_expList.columnContextMenu.Items.Clear()
            Next
        End Sub

        ''' <summary>
        ''' Sets up an ExpListColumn and sets the ExpListItems text for that column
        ''' </summary>
        ''' <param name="column"></param>
        ''' <remarks></remarks>
        Private Sub SetUpColumn(ByVal column As ExpListColumn)
            'Check that the ExpList property has been set for this column
            'This is required for designer support
            If column.ExpList Is Nothing Then
                column.ExpList = m_expList
            End If

            'Disable the ContextMenu Item if this is the first column
            column.contextMenuItem.Enabled = True
            column.contextMenuItem.Checked = True
            If column.Index = 0 Then
                column.contextMenuItem.Enabled = False
            End If

            m_expList.sortMenu.DropDownItems.Insert(Me.IndexOf(column), column.sortMenuItem)

            'If Clear is called and then a standard column is added again
            'the ContextMenuItem will have been removed...
            If m_expList.columnContextMenu.Items.IndexOf(column.contextMenuItem) < 0 Then
                m_expList.columnContextMenu.Items.Add(column.contextMenuItem)
            End If

            'Loop through all the Items and add a SubItem for
            'the new column.
            Dim listItem As ExpListItem
            For Each listItem In m_expList.Items
                listItem.SubItems.Insert(Me.IndexOf(column), New ExpListItem.ListViewSubItem(listItem, ""))
            Next

            'Set the text
            Dim item As ExpListItem
            For Each item In m_expList.Items
                m_expList.SetItemText(item, column)
            Next

        End Sub

    End Class

#End Region

End Class