Option Strict On

Friend Class ExpListComparer
    Implements IComparer

    Private m_column As ExpListColumn
    Private m_sortOrder As SortOrder
    Private m_isMyComputer As Boolean
    Private m_grouped As Boolean

    Friend Sub New(ByVal column As ExpListColumn, ByVal sortOrder As SortOrder, _
            Optional ByVal isMyComputer As Boolean = False, Optional ByVal grouped As Boolean = False)
        'Store the desired column, sort order and the two sort flags.
        m_column = column
        m_sortOrder = sortOrder
        m_isMyComputer = isMyComputer
        m_grouped = grouped
    End Sub

    ' Compare the items in the appropriate column
    ' for objects x and y.
    Friend Function Compare(ByVal x As Object, ByVal y As _
        Object) As Integer Implements System.Collections.IComparer.Compare

        Dim listItemX As ExpListItem = DirectCast(x, _
            ExpListItem)
        Dim listItemY As ExpListItem = DirectCast(y, _
            ExpListItem)

        'Check if we are sorting by type in MyComputer, if so
        'then just rearrange things a little bit...
        If m_isMyComputer And m_grouped And m_column.ColumnType = ExpListColumnType.Type Then
            Return CompareMyComputer(listItemX.CShItem, listItemY.CShItem)
        End If

        Dim stringX As String = listItemX.SubItems(m_column.Index).Text
        Dim stringY As String = listItemY.SubItems(m_column.Index).Text

        'If the string are the same then return 0
        If stringX = stringY Then Return 0

        'Make sure empty strings appear at the bottom
        If stringX = "" And Not stringY = "" Then
            If m_sortOrder = SortOrder.Ascending Then
                Return 1
            Else
                Return -1
            End If
        ElseIf stringY = "" And Not stringX = "" Then
            If m_sortOrder = SortOrder.Ascending Then
                Return -1
            Else
                Return 1
            End If
        End If
        'Seperate files from folders
        If Not m_grouped Then
            If listItemX.CShItem.IsFolder And Not listItemY.CShItem.IsFolder Then
                If m_sortOrder = SortOrder.Ascending Then
                    Return -1
                Else
                    Return 1
                End If
            ElseIf listItemY.CShItem.IsFolder And Not listItemX.CShItem.IsFolder Then
                If m_sortOrder = SortOrder.Ascending Then
                    Return 1
                Else
                    Return -1
                End If
            End If
        End If
        'Items are either both folders or both files
        'or we aren't seperating the folders
        Select Case m_column.ColumnType
            Case ExpListColumnType.Size
                If m_sortOrder = SortOrder.Ascending Then
                    'just return 1 or -1 to avoid an overflow converting
                    'the length to an int to return
                    If (listItemX.CShItem.Length - listItemY.CShItem.Length) < 0 Then
                        Return -1
                    Else : Return 1
                    End If
                Else
                    If (listItemY.CShItem.Length - listItemX.CShItem.Length) < 0 Then
                        Return -1
                    Else : Return 1
                    End If
                End If

            Case ExpListColumnType.CustomDate, ExpListColumnType.Modified, ExpListColumnType.Created

                Try
                    Dim dateX As Date = CDate(stringX)
                    Dim dateY As Date = CDate(stringY)
                    If m_sortOrder = SortOrder.Ascending Then
                        Return Date.Compare(dateX, dateY)
                    Else
                        Return Date.Compare(dateY, dateX)
                    End If
                Catch ex As Exception
                    Throw New Exception("Invalid Date Format.")
                End Try

            Case ExpListColumnType.CustomNumeric
                Dim value As Double
                Try
                    'If there is an error converting then just compare the strings
                    value = CDbl(stringX) - CDbl(stringY)
                Catch ex As Exception
                    Throw New Exception("Error comparing " & stringX & " and " & stringY & ".")
                End Try
                If m_sortOrder = SortOrder.Ascending Then
                    If value >= 0 Then
                        Return 1
                    Else
                        Return -1
                    End If
                Else
                    If value >= 0 Then
                        Return -1
                    Else
                        Return 1
                    End If
                End If
        End Select
        If m_sortOrder = SortOrder.Ascending Then
            Return String.Compare(stringX, stringY)
        Else
            Return String.Compare(stringY, stringX)
        End If

    End Function

    Private Function CompareMyComputer(ByVal itemX As CShItem, ByVal itemY As CShItem) As Integer

        Dim typeX As Integer
        Dim typeY As Integer

        If itemY.IsRemovable Then
            typeY = 3
        ElseIf itemY.IsDisk Then
            If itemY.IsNetworkDrive Then
                typeY = 4
            Else
                typeY = 2
            End If
        ElseIf itemY.IsFolder Then
            If itemY.TypeName = CShItem.strSystemFolder Then
                typeY = 5
            Else
                typeY = 1
            End If
        Else
            typeY = 5
        End If

        If itemX.IsRemovable Then
            typeX = 3
        ElseIf itemX.IsDisk Then
            If itemX.IsNetworkDrive Then
                typeX = 4
            Else
                typeX = 2
            End If
        ElseIf itemX.IsFolder Then
            If itemX.TypeName = CShItem.strSystemFolder Then
                typeX = 5
            Else
                typeX = 1
            End If
        Else
            typeX = 5
        End If

        Dim result As Integer = typeX - typeY
        Select Case m_sortOrder
            Case SortOrder.Ascending
                If result = 0 Then
                    Return String.Compare(itemX.TypeName, itemY.TypeName)
                End If
                Return result
            Case Else
                If result = 0 Then
                    Return String.Compare(itemY.TypeName, itemX.TypeName)
                End If
                Return (result * -1)
        End Select

    End Function

End Class

