Imports System.Runtime.InteropServices

Partial Class ExpList

#Region " ExpListItemEventArgs "
''' <summary>
''' EventArgs used on activation of an ExpListItem
''' </summary>
''' <remarks></remarks>
Public Class ExpListItemEventArgs
    Inherits EventArgs

    Private m_shItem As CShItem
    ''' <summary>
    ''' The CShItem associated with the ExpListItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property CShItem() As CShItem
        Get
            Return m_shItem
        End Get
    End Property

    Private m_largeIcon As Icon
    ''' <summary>
    ''' The Large Icon associated with the ExpListItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property LargeIcon() As Icon
        Get
            Return m_largeIcon
        End Get
    End Property

    Private m_smallIcon As Icon
    ''' <summary>
    ''' The Small Icon associated with the ExpListItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property SmallIcon() As Icon
        Get
            Return m_smallIcon
        End Get
    End Property

    Private m_iconIndex As Integer
    ''' <summary>
    ''' The Index of the associated Icons in the SystemImageList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IconIndex() As Integer
        Get
            Return m_iconIndex
        End Get
    End Property

    Private m_item As ExpListItem
    ''' <summary>
    ''' The ExpListItem associated with the Event
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Item() As ExpListItem
        Get
            Return m_item
        End Get
    End Property

    Friend Sub New(ByVal item As ExpListItem)
        m_item = item
        m_shItem = item.CShItem
        m_iconIndex = item.ImageIndex
        m_largeIcon = SystemImageListManager.GetIcon(item.ImageIndex)
        m_smallIcon = SystemImageListManager.GetIcon(item.ImageIndex, True)
    End Sub

End Class

#End Region

#Region " ExpListSortEventArgs "
''' <summary>
''' EventArgs returned before and after sorting an ExpList
''' </summary>
''' <remarks></remarks>
Public Class ExpListSortEventArgs
    Inherits EventArgs

    Private m_column As ExpListColumn
    ''' <summary>
    ''' The column used to sort the ExpListItems
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Column() As ExpListColumn
        Get
            Return m_column
        End Get
    End Property

    Private m_sortOrder As Windows.Forms.SortOrder
    ''' <summary>
    ''' The sort order being used
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property SortOrder() As Windows.Forms.SortOrder
        Get
            Return m_sortOrder
        End Get
    End Property

    Friend Sub New(ByVal sortOrder As Windows.Forms.SortOrder, ByVal column As ExpListColumn)
        m_sortOrder = sortOrder
        m_column = column
    End Sub

End Class

#End Region

#Region " ExpListGroupEventArgs "
''' <summary>
''' EventArgs returned for the ExpList.Group Event
''' </summary>
''' <remarks></remarks>
Public Class ExpListGroupEventArgs
    Inherits EventArgs

    Private m_column As ExpListColumn
    ''' <summary>
    ''' The column used to group the ExpListItems
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Column() As ExpListColumn
        Get
            Return m_column
        End Get
    End Property

    Friend Sub New(ByVal column As ExpListColumn)
        m_column = column
    End Sub

End Class

#End Region

#Region " ExpListItemsAddedEventArgs "
''' <summary>
''' EventArgs for when one or more Items are added to the ExpList
''' </summary>
''' <remarks></remarks>
Public Class ExpListItemsAddedEventArgs
    Inherits EventArgs

    Private m_items() As ExpListItem
    ''' <summary>
    ''' An ArrayList of the ExpListItems that have been added
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Items() As ExpListItem()
        Get
            Return m_items
        End Get
    End Property

    Friend Sub New(ByVal items() As ExpListItem)
        m_items = items
    End Sub

End Class

#End Region

End Class
