Option Strict On
Imports ExplorerControls.ExpList
Imports System.ComponentModel
Imports System.Runtime.InteropServices

''' <summary>
''' The Item Class for the ExpList Control
''' </summary>
''' <remarks></remarks>
<Browsable(False), _
 Guid("F34715C9-0720-4794-BD6F-64D222F9A67E")> _
Public Class ExpListItem
    Inherits ListViewItem

#Region " Fields And Properties "

    Protected noThumbnail As Boolean

    Private m_item As CShItem
    ''' <summary>
    ''' The CShItem associated with the ExpListItem
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property CShItem() As CShItem
        Get
            Return m_item
        End Get
        Set(ByVal value As CShItem)
            m_item = value
            Me.ImageIndex = SystemImageListManager.GetIconIndex(m_item)
            m_thumbnail = Nothing
        End Set
    End Property

    ''' <summary>
    ''' Hide the inherited ListView Property to force the user to use the ExpList Property
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shadows ReadOnly Property ListView() As ExpList
        Get
            Return CType(MyBase.ListView, ExpList)
        End Get
    End Property

    ''' <summary>
    ''' The parent ExpList Control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ExpList() As ExpList
        Get
            Return CType(MyBase.ListView, ExpList)
        End Get
    End Property

    Private m_state As ExpListItemState = ExpListItemState.Normal
    ''' <summary>
    ''' The State of the ExpListItem. One of the values of the ExpListItemState Enum
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property State() As ExpListItemState
        Get
            Return m_state
        End Get
        Set(ByVal Value As ExpListItemState)
            If IsNothing(Me.ExpList) Then Exit Property
            Dim lv As New LVITEM
            lv.mask = LVIF_STATE
            Select Case Value
                Case ExpListItemState.Normal
                    With lv
                        .state = 0 'False
                        'If the Item isn't a hidden folder
                        'then remove the "Cut" state.
                        If Not m_item.IsHidden Then
                            .stateMask = LVIS_CUT
                        End If
                    End With
                Case ExpListItemState.Cut
                    With lv
                        .state = -1 'True
                        .stateMask = LVIS_CUT
                    End With
            End Select
            'Set the state if the call is successful
            If SendMessage(Me.ExpList.Handle, LVM_SETITEMSTATE, Me.ExpList.Items.IndexOf(Me), lv) Then
                m_state = Value
            End If
        End Set
    End Property


    Private m_thumbnail As Image
    ''' <summary>
    ''' The Thumbnail for the associated CShItem.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public Property Thumbnail() As Image
        Get
            If noThumbnail Then
                Return Nothing
            ElseIf IsNothing(m_thumbnail) Then
                GetThumbnail()
            End If
            Return m_thumbnail
        End Get
        Set(ByVal value As Image)
            m_thumbnail = value
        End Set
    End Property

#End Region

#Region " Constructors "
    ''' <summary>
    ''' Creates a new ExpListItem
    ''' </summary>
    ''' <param name="item">The CShItem to be associated with the ExpListItem</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal item As CShItem)
        MyBase.New()

        'Store the variables
        m_item = item
        Me.ImageIndex = SystemImageListManager.GetIconIndex(m_item)
    End Sub

    ''' <summary>
    ''' For COM support
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.new()
    End Sub

#End Region

    ''' <summary>
    ''' Gets the Thumbnail image for the associated CShItem
    ''' </summary>
    ''' <returns>A 100px by 100px Image</returns>
    ''' <remarks></remarks>
    Public Function GetThumbnail() As Image
        m_thumbnail = ExplorerControls.Thumbnail.ExtractThumbNail(Me.CShItem)
        'Set flag if this item doesn't have a thumbnail
        If IsNothing(m_thumbnail) Then noThumbnail = True
        Return m_thumbnail
    End Function

    ''' <summary>
    ''' Creates an identical copy of this item
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Function Clone() As Object
        Dim item As New ExpListItem(m_item)
        item.Thumbnail = Me.Thumbnail
        Return item
    End Function

End Class
