Imports System.Runtime.InteropServices
Imports System.ComponentModel

<ClassInterface(ClassInterfaceType.None)> _
Public Class FlatButton
    Inherits Button

#Region " Fields And Properties "

    Private m_backColor As Color
    Private m_focus As Boolean = False
    Private m_mouseEntered As Boolean = False

    Private m_highlightColor As Color = Color.Gainsboro
    ''' <summary>
    ''' BackColor for the button when it has the focus
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("BackColor for the button when it has the focus."), _
    DefaultValue(GetType(Color), "Gainsboro")> _
    Public Property HighlightColor() As Color
        Get
            Return m_highlightColor
        End Get
        Set(ByVal Value As Color)
            m_highlightColor = Value
        End Set
    End Property

    Private m_hoverColor As Color = System.Drawing.Color.Silver
    ''' <summary>
    ''' BackColor for the button when the mouse is hovering over it
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("BackColor for the button when the mouse is hovering over it."), _
    DefaultValue(GetType(Color), "Silver")> _
    Public Property HoverColor() As Color
        Get
            Return m_hoverColor
        End Get
        Set(ByVal Value As Color)
            m_hoverColor = Value
        End Set
    End Property

    Private m_highlightHover As Color = System.Drawing.Color.DarkGray
    ''' <summary>
    ''' BackColor for the button when it has the focus and the mouse is hovering over it
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("BackColor for the button when it has the focus and the mouse is hovering over it."), _
    DefaultValue(GetType(Color), "DarkGray")> _
    Public Property HighlightHoverColor() As Color
        Get
            Return m_highlightHover
        End Get
        Set(ByVal Value As Color)
            m_highlightHover = Value
        End Set
    End Property

#End Region

    Public Sub New()
        MyBase.New()
        m_backColor = Me.BackColor
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        MyBase.Dispose(disposing)
    End Sub

#Region " Event Handlers "

#Region " OnPaint "

    Protected Overrides Sub OnPaint(ByVal pevent As System.Windows.Forms.PaintEventArgs)

        If Not m_focus And Not m_mouseEntered Then
            'Draw the background without a border
            pevent.Graphics.FillRectangle(New SolidBrush(Me.BackColor), New Rectangle(0, 0, Me.Width, Me.Height))
        Else
            'Draw Background with a border
            pevent.Graphics.DrawRectangle(New Pen(Me.ForeColor, 0), New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
            pevent.Graphics.FillRectangle(New SolidBrush(Me.BackColor), New Rectangle(1, 1, Me.Width - 2, Me.Height - 2))
        End If
        If Not IsNothing(Me.Image) Then
            If Me.Enabled Then
                'Check the image alignment - override it if it is not 
                'TopCenter or MiddleCenter
                'Always defining the entire rectangle for the image increases performance
                Select Case Me.ImageAlign
                    Case ContentAlignment.BottomCenter
                        pevent.Graphics.DrawImage(Me.Image, CInt((Me.Width - Me.Image.Width) / 2), Me.Height - Me.Image.Height - 5, _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.BottomLeft
                        pevent.Graphics.DrawImage(Me.Image, 5, Me.Height - Me.Image.Height - 5, _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.BottomRight
                        pevent.Graphics.DrawImage(Me.Image, Me.Width - Me.Image.Width - 5, Me.Height - Me.Image.Height - 5, _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.MiddleCenter
                        pevent.Graphics.DrawImage(Me.Image, CInt((Me.Width - Me.Image.Width) / 2), CInt((Me.Height - Me.Image.Height) / 2), _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.MiddleLeft
                        pevent.Graphics.DrawImage(Me.Image, 5, CInt((Me.Height - Me.Image.Height) / 2), _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.MiddleRight
                        pevent.Graphics.DrawImage(Me.Image, Me.Width - Me.Image.Width - 5, CInt((Me.Height - Me.Image.Height) / 2), _
                            Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.TopCenter
                        pevent.Graphics.DrawImage(Me.Image, CInt((Me.Width - Me.Image.Width) / 2), 5, Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.TopLeft
                        pevent.Graphics.DrawImage(Me.Image, 5, 5, Me.Image.Width, Me.Image.Height)
                    Case ContentAlignment.TopRight
                        pevent.Graphics.DrawImage(Me.Image, Me.Width - Me.Image.Width - 5, 5, Me.Image.Width, Me.Image.Height)
                    Case Else
                        'Override
                        Me.ImageAlign = ContentAlignment.MiddleCenter
                        Me.OnPaint(pevent)
                End Select
            Else
                'Draw a grayscale representation using a ColorMatrix
                Dim ia As New Imaging.ImageAttributes
                Dim cm As New Imaging.ColorMatrix
                cm.Matrix00 = 1 / 3.0F
                cm.Matrix01 = 1 / 3.0F
                cm.Matrix02 = 1 / 3.0F
                cm.Matrix10 = 1 / 3.0F
                cm.Matrix11 = 1 / 3.0F
                cm.Matrix12 = 1 / 3.0F
                cm.Matrix20 = 1 / 3.0F
                cm.Matrix21 = 1 / 3.0F
                cm.Matrix22 = 1 / 3.0F
                ia.SetColorMatrix(cm)
                Select Case Me.ImageAlign
                    Case ContentAlignment.TopCenter
                        pevent.Graphics.DrawImage(Me.Image, New Rectangle(CInt((Me.Width - Me.Image.Width) / 2), 5, Me.Image.Width, Me.Image.Height), _
                                    0, 0, Me.Image.Width, Me.Image.Height, GraphicsUnit.Pixel, ia)
                    Case ContentAlignment.MiddleCenter
                        pevent.Graphics.DrawImage(Me.Image, New Rectangle(CInt((Me.Width - Me.Image.Width) / 2), CInt((Me.Height - Me.Image.Height) / 2), _
                                    Me.Image.Width, Me.Image.Height), 0, 0, Me.Image.Width, Me.Image.Height, GraphicsUnit.Pixel, ia)
                    Case Else
                        'Override
                        Me.ImageAlign = ContentAlignment.MiddleCenter
                        Me.OnPaint(pevent)
                End Select
            End If
        End If

        Dim b As Brush
        If Me.Enabled Then
            b = New SolidBrush(Me.ForeColor)
        Else
            'Haven't figured out how to draw sunken text yet
            b = New SolidBrush(SystemColors.ControlDark)
        End If
        If Not IsNothing(Me.Text) Then
            'Write the text
            Dim txtSizeF As SizeF = pevent.Graphics.MeasureString(Me.Text, Me.Font)
            Dim singleLine As Boolean
            If CInt(txtSizeF.Width) < Me.Width Then
                singleLine = True
            End If
            'Create a Format for the text
            Dim sf As StringFormat = StringFormat.GenericTypographic
            sf.Trimming = StringTrimming.Word
            sf.FormatFlags = StringFormatFlags.LineLimit
            Dim paddingBottom As Integer = 3
            Dim paddingTop As Integer = 3

            Select Case Me.TextAlign
                Case ContentAlignment.BottomCenter
                    sf.Alignment = StringAlignment.Center
                    sf.LineAlignment = StringAlignment.Far
                    If singleLine Then paddingBottom = CInt(txtSizeF.Height)
                Case ContentAlignment.BottomLeft
                    sf.Alignment = StringAlignment.Near
                    sf.LineAlignment = StringAlignment.Far
                    If singleLine Then paddingBottom = CInt(txtSizeF.Height)
                Case ContentAlignment.BottomRight
                    sf.Alignment = StringAlignment.Far
                    sf.LineAlignment = StringAlignment.Far
                    If singleLine Then paddingBottom = CInt(txtSizeF.Height)
                Case ContentAlignment.MiddleCenter
                    sf.Alignment = StringAlignment.Center
                    sf.LineAlignment = StringAlignment.Center
                Case ContentAlignment.MiddleLeft
                    sf.Alignment = StringAlignment.Near
                    sf.LineAlignment = StringAlignment.Center
                Case ContentAlignment.MiddleRight
                    sf.Alignment = StringAlignment.Far
                    sf.LineAlignment = StringAlignment.Center
                Case ContentAlignment.TopCenter
                    sf.Alignment = StringAlignment.Center
                    sf.LineAlignment = StringAlignment.Near
                    If singleLine Then paddingTop = CInt(txtSizeF.Height)
                Case ContentAlignment.TopLeft
                    sf.Alignment = StringAlignment.Near
                    sf.LineAlignment = StringAlignment.Near
                    If singleLine Then paddingTop = CInt(txtSizeF.Height)
                Case ContentAlignment.TopRight
                    sf.Alignment = StringAlignment.Far
                    sf.LineAlignment = StringAlignment.Near
                    If singleLine Then paddingTop = CInt(txtSizeF.Height)
            End Select
            Dim rect As New Rectangle(3, paddingTop, Me.Bounds.Width - 6, _
                            Me.Bounds.Height - paddingBottom - paddingTop)
            pevent.Graphics.DrawString(Me.Text, Me.Font, b, rect, sf)
        End If
        b.Dispose()
    End Sub

#End Region

    Private Sub FlatButton_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.MouseEnter
        m_mouseEntered = True
        If m_focus Then
            Me.BackColor = m_highlightHover
        Else
            Me.BackColor = m_hoverColor
        End If
    End Sub

    Private Sub FlatButton_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.MouseLeave
        m_mouseEntered = False
        If m_focus Then
            Me.BackColor = m_highlightColor
        Else
            Me.BackColor = m_backColor
        End If
    End Sub

    Private Sub FlatButton_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.GotFocus
        m_focus = True
        If m_mouseEntered Then
            Me.BackColor = m_highlightHover
        Else
            Me.BackColor = m_highlightColor
        End If
    End Sub

    Private Sub FlatButton_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.LostFocus
        m_focus = False
        If m_mouseEntered Then
            Me.BackColor = m_hoverColor
        Else
            Me.BackColor = m_backColor
        End If
    End Sub

#End Region

End Class
