Imports System.Drawing
Imports System.Runtime.InteropServices
Imports System.Text
Imports ExplorerControls.ShellDll

''' <summary>
''' Class for retrieving the Thumbnail associated with a file
''' </summary>
''' <remarks></remarks>
<ClassInterface(ClassInterfaceType.None)> _
Public Class Thumbnail
    Private Const GUID_IEXTRACTIMAGE As String = "BB2E617C-0920-11d1-9A0B-00C04FC2D6C1"

    ''' <summary>
    ''' Extracts a standard sized Thumbnail for the specified file
    ''' </summary>
    ''' <param name="file">The CShItem associated with the file</param>
    ''' <returns>A standard sized Thumbnail Image (96px by 96px)</returns>
    ''' <remarks></remarks>
    Public Shared Function ExtractThumbNail(ByVal file As CShItem) As Bitmap
        Return ExtractThumbNail(file, New Size(96, 96))
    End Function

    ''' <summary>
    ''' Extracts a Thumbnail Image in the specified size for the specified file
    ''' </summary>
    ''' <param name="file">The CShItem associated with the file</param>
    ''' <param name="imageSize">The desired size of the image</param>
    ''' <returns>A Thumbnail Image in the specified size</returns>
    ''' <remarks></remarks>
    Public Shared Function ExtractThumbNail(ByVal file As CShItem, ByVal imageSize As Size) As Bitmap

        Dim thumbnailImage As Bitmap = Nothing

        Dim hBmp As IntPtr = IntPtr.Zero
        Dim extractImage As IExtractImage = Nothing
        Dim iunk As IUnknown = Nothing
        Dim pidl As IntPtr = IntPtr.Zero
        Dim folder As IShellFolder = CShItem.GetParentOf(file.PIDL, pidl)
        Try
            Dim prgf As Integer = 0
            Dim iidExtractImage As Guid = New Guid(GUID_IEXTRACTIMAGE)
            Dim hr As Integer = folder.GetUIObjectOf(IntPtr.Zero, 1, New IntPtr() {pidl}, iidExtractImage, prgf, iunk)
            If Not hr = 0 Then
                Marshal.ThrowExceptionForHR(hr)
            End If
            extractImage = CType(iunk, IExtractImage)

            If (extractImage IsNot Nothing) Then
                Dim location As New StringBuilder(260, 260)

                Dim priority As Integer = 0
                Dim requestedColorDepth As Integer = 32
                Dim uFlags As Integer = IEIFLAG.IEIFLAG_ASPECT

                extractImage.GetLocation(location, location.Capacity, priority, imageSize, requestedColorDepth, uFlags)

                extractImage.Extract(hBmp)
                If (hBmp <> IntPtr.Zero) Then
                    thumbnailImage = Bitmap.FromHbitmap(hBmp)
                End If
                Return thumbnailImage
            Else
                Return Nothing
            End If
        Catch ex As Exception
            Return Nothing
        Finally
            If (hBmp <> IntPtr.Zero) Then DeleteObject(hBmp)
            If (iunk IsNot Nothing) Then
                Marshal.ReleaseComObject(iunk)
                iunk = Nothing
            End If
            If (folder IsNot Nothing) Then
                Marshal.ReleaseComObject(folder)
                folder = Nothing
            End If
            If (extractImage IsNot Nothing) Then
                Marshal.ReleaseComObject(extractImage)
                extractImage = Nothing
            End If
            pidl = IntPtr.Zero
        End Try
    End Function

    <DllImport("gdi32", CharSet:=CharSet.Auto)> _
    Private Shared Function DeleteObject(ByVal hObject As IntPtr) As Integer
    End Function

End Class

<ComImportAttribute(), _
GuidAttribute("BB2E617C-0920-11d1-9A0B-00C04FC2D6C1"), _
InterfaceTypeAttribute(ComInterfaceType.InterfaceIsIUnknown)> _
Friend Interface IExtractImage

    Sub GetLocation(<Out(), MarshalAs(UnmanagedType.LPWStr)> ByVal pszPathBuffer As StringBuilder, _
    ByVal cch As Integer, ByRef pdwPriority As Integer, ByRef prgSize As Size, _
    ByVal dwRecClrDepth As Integer, ByRef pdwFlags As Integer)

    Sub Extract(<Out()> ByRef phBmpThumbnail As IntPtr)

End Interface

Friend Enum IEIFLAG
    IEIFLAG_ASYNC = &H1     ' ask the extractor if it supports ASYNC extract (free threaded)
    IEIFLAG_CACHE = &H2      'returned from the extractor if it does NOT cache the thumbnail
    IEIFLAG_ASPECT = &H4      ' passed to the extractor to beg it to render to the aspect ratio of the supplied rect
    IEIFLAG_OFFLINE = &H8     ' if the extractor shouldn't hit the net to get any content neede for the rendering
    IEIFLAG_GLEAM = &H10     'does the image have a gleam ? this will be returned if it does
    IEIFLAG_SCREEN = &H20      ' render as if for the screen  (this is exlusive with IEIFLAG_ASPECT )
    IEIFLAG_ORIGSIZE = &H40      ' render to the approx size passed, but crop if neccessary
    IEIFLAG_NOSTAMP = &H80      ' returned from the extractor if it does NOT want an icon stamp on the thumbnail
    IEIFLAG_NOBORDER = &H100      'returned from the extractor if it does NOT want an a border around the thumbnail
    IEIFLAG_QUALITY = &H200      ' passed to the Extract method to indicate that a slower, higher quality image is desired, re-compute the thumbnail
End Enum

