'Option Strict On

Imports System.IO
Imports System.ComponentModel
Imports system.Globalization
Imports System.Threading
Imports System.Resources
Imports System.Runtime.InteropServices
Imports Microsoft.Win32
Imports System.Drawing.Design

<ClassInterface(ClassInterfaceType.None), DefaultProperty("SelectedItems"), DefaultEvent("SelectedFolderChanged")> _
Public Class Browser
    Inherits System.Windows.Forms.UserControl

#Region " Events "

    'Declare the Delegates
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub SelectedListIndexChangedEventHandler(ByVal sender As Object, ByVal e As EventArgs)
    <Browsable(False), EditorBrowsable(EditorBrowsableState.Never)> _
    Public Delegate Sub SelectedFolderChangedEventHandler(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' Fires when the selected index in the ExpList changes
    ''' </summary>
    ''' <remarks></remarks>
    Public Event SelectedListIndexChanged As SelectedListIndexChangedEventHandler
    ''' <summary>
    ''' Fires when the selected item in the ExpCombo changes
    ''' </summary>
    ''' <remarks></remarks>
    Public Event SelectedFolderChanged As SelectedFolderChangedEventHandler
    ''' <summary>
    ''' Fires when an ExpListItem that represents a file is activated
    ''' </summary>
    ''' <remarks></remarks>
    Public Event FileActivate As ExpList.FileActivateEventHandler
    ''' <summary>
    ''' Fires when an ExpListItem that represents a folder is activated
    ''' </summary>
    ''' <remarks></remarks>
    Public Event FolderActivate As ExpList.FolderActivateEventHandler
    ''' <summary>
    ''' Fires directly before the ExpList is sorted
    ''' </summary>
    ''' <remarks></remarks>
    Public Event BeforeSort As ExpList.BeforeSortEventHandler
    ''' <summary>
    ''' Fires directly after the ExpList is sorted
    ''' </summary>
    ''' <remarks></remarks>
    Public Event AfterSort As ExpList.AfterSortEventHandler
    ''' <summary>
    ''' Fires directly before the ExpList is grouped
    ''' </summary>
    ''' <remarks></remarks>
    Public Event Group As ExpList.GroupEventHandler
    ''' <summary>
    ''' Fires when a new folder is created in the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Public Event NewFolder As ExpList.NewFolderEventHandler
    ''' <summary>
    ''' Fires directly after new Items have been added to the ExpList
    ''' </summary>
    ''' <remarks></remarks>
    Public Event ItemsAdded As ExpList.ItemsAddedEventHandler
#End Region

#Region " Fields and Properties "

#Region "       ExpCombo "

    ''' <summary>
    ''' Items collection of the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property ExpComboItems() As ComboBox.ObjectCollection
        Get
            Return expCB.Items
        End Get
    End Property

    ''' <summary>
    ''' Selected item in the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(CType(Nothing, Object))> _
    Public Property SelectedComboItem() As ExpComboItem
        Get
            Return Me.expCB.SelectedItem
        End Get
        Set(ByVal Value As ExpComboItem)
            Me.expCB.SelectedItem = Value
        End Set
    End Property

    ''' <summary>
    ''' Selected index of the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(-1)> _
    Public Property SelectedComboIndex() As Integer
        Get
            Return Me.expCB.SelectedIndex
        End Get
        Set(ByVal Value As Integer)
            Me.expCB.SelectedIndex = Value
        End Set
    End Property

    ''' <summary>
    ''' Root Directory for the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpCombo"), Description("Root Directory for the ExpCombo."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property RootDirectory() As ExpTree.StartDir
        Get
            Return expCB.RootDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            expCB.RootDirectory = Value
            'Clear navigation History
            m_backList = New ArrayList
            m_fowardList = New ArrayList
            CheckButtons()
            'Clear the list 
            Me.expLV.Items.Clear()
        End Set
    End Property

    ''' <summary>
    ''' Startup Directory for the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpCombo"), Description("Startup Directory for the ExpCombo."), _
    DefaultValue(GetType(ExpTree.StartDir), "None")> _
    Public Property StartUpDirectory() As ExpTree.StartDir
        Get
            Return expCB.StartUpDirectory
        End Get
        Set(ByVal Value As ExpTree.StartDir)
            expCB.StartUpDirectory = Value
        End Set
    End Property

    ''' <summary>
    ''' Maximum number of drop-down items shown in the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpCombo"), Description("Maximum number of drop-down items shown in the ExpCombo."), _
    DefaultValue(14)> _
    Public Property MaxComboDropDownItems() As Integer
        Get
            Return expCB.MaxDropDownItems
        End Get
        Set(ByVal Value As Integer)
            expCB.MaxDropDownItems = Value
        End Set
    End Property

    ''' <summary>
    ''' ExpCombo Item Height
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpCombo"), Description("ExpCombo Item Height."), _
    DefaultValue(18)> _
    Public Property ComboItemHeight() As Integer
        Get
            Return expCB.ItemHeight
        End Get
        Set(ByVal Value As Integer)
            expCB.ItemHeight = Value
        End Set
    End Property

    ''' <summary>
    ''' Determines whether to show the full path in the ExpCombo edit area
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpCombo"), Description("Whether to show the full path in the ExpCombo edit area."), _
    DefaultValue(False)> _
    Public Property DisplayPath() As Boolean
        Get
            Return expCB.DisplayPath
        End Get
        Set(ByVal Value As Boolean)
            expCB.DisplayPath = Value
        End Set
    End Property

    ''' <summary> 
    ''' The Style of the ExpCombo
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("The Style of the ExpCombo."), _
    DefaultValue(GetType(ExpComboStyle), "CurrentFolder")> _
    Public Property Style() As ExpComboStyle
        Get
            Return expCB.Style
        End Get
        Set(ByVal value As ExpComboStyle)
            expCB.Style = value
        End Set
    End Property

    ''' <summary> 
    ''' The Offset size for the folder heirachy
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Appearance"), Description("The Offset size for the folder heirachy."), _
    DefaultValue(10)> _
    Public Property Offset() As Integer
        Get
            Return expCB.Offset
        End Get
        Set(ByVal value As Integer)
            expCB.Offset = value
        End Set
    End Property

#End Region

#Region "       ExpList "

    ''' <summary>
    ''' The items collection in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property ExpListItems() As ExpList.ExpListItemCollection
        Get
            Return expLV.Items
        End Get
    End Property

    ''' <summary>
    ''' The selected items in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SelectedListItems() As ExpList.SelectedExpListItemCollection
        Get
            Return Me.expLV.SelectedItems
        End Get
    End Property

    ''' <summary>
    ''' The selected indices of the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SelectedListIndicies() As ExpList.SelectedIndexCollection
        Get
            Return Me.expLV.SelectedIndices
        End Get
    End Property

    ''' <summary>
    ''' The columns in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior")> _
    <EditorAttribute(GetType(System.ComponentModel.Design.CollectionEditor), GetType(UITypeEditor))> _
    <DesignerSerializationVisibility(DesignerSerializationVisibility.Content)> _
    Public ReadOnly Property Columns() As ExpList.ExpListColumnCollection
        Get
            Return expLV.Columns
        End Get
    End Property

    ''' <summary>
    ''' The groups collection in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property ExpListGroups() As ListViewGroupCollection
        Get
            Return expLV.Groups
        End Get
    End Property

    ''' <summary>
    ''' The current sort order of the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SortOrder() As Windows.Forms.SortOrder
        Get
            Return expLV.SortOrder
        End Get
    End Property

    Private m_sortColumn As ExpListColumn
    ''' <summary>
    ''' The current sorting column of the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False)> _
    Public ReadOnly Property SortColumn() As ExpListColumn
        Get
            Return expLV.SortColumn
        End Get
    End Property

    ''' <summary>
    ''' Determines whether the ExpList is currently grouped
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(False)> _
    Public Property Grouped() As Boolean
        Get
            Return expLV.Grouped
        End Get
        Set(ByVal Value As Boolean)
            expLV.Grouped = Value
        End Set
    End Property

    ''' <summary>
    ''' Open files in ExpList when activated
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Open files in ExpList when activated."), _
    DefaultValue(True)> _
        Public Property FileOpenOnActivate() As Boolean
        Get
            Return expLV.FileOpenOnActivate
        End Get
        Set(ByVal Value As Boolean)
            expLV.FileOpenOnActivate = Value
        End Set
    End Property

    ''' <summary>
    ''' Open folders in ExpList when activated
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Open folders in ExpList when activated."), _
    DefaultValue(True)> _
        Public Property FolderOpenOnActivate() As Boolean
        Get
            Return expLV.FolderOpenOnActivate
        End Get
        Set(ByVal Value As Boolean)
            expLV.FolderOpenOnActivate = Value
        End Set
    End Property

    ''' <summary>
    ''' Filter sting for file types to be shown in the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Filter sting for file types to be shown in the ExpList."), _
    DefaultValue(CType(Nothing, Object))> _
    Public Property Filter() As String()
        Get
            Return expLV.Filter
        End Get
        Set(ByVal Value As String())
            expLV.Filter = Value
        End Set
    End Property

    ''' <summary>
    ''' ExpList Multiselect
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("ExpList Multiselect."), _
    DefaultValue(True)> _
    Public Property Multiselect() As Boolean
        Get
            Return expLV.MultiSelect
        End Get
        Set(ByVal Value As Boolean)
            expLV.MultiSelect = Value
        End Set
    End Property

    ''' <summary>
    ''' ContextMenu for the ExpList
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("ContextMenu for the ExpList.")> _
    Public Shadows Property ContextMenu() As ContextMenu
        Get
            Return expLV.ContextMenu
        End Get
        Set(ByVal Value As ContextMenu)
            expLV.ContextMenu = Value
        End Set
    End Property

    ''' <summary>
    ''' Default ExpList ContextMenu
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Default ExpList ContextMenuStrip.")> _
    Public ReadOnly Property DefaultContextMenu() As ContextMenuStrip
        Get
            Return expLV.DefaultContextMenu
        End Get
    End Property

    ''' <summary>
    ''' Determines whether ExpList columns can be reordered by the user
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Determines whether ExpList columns can be reordered by the user."), _
    DefaultValue(True)> _
    Public Property AllowColumnReorder() As Boolean
        Get
            Return expLV.AllowColumnReorder
        End Get
        Set(ByVal Value As Boolean)
            expLV.AllowColumnReorder = Value
        End Set
    End Property

    ''' <summary>
    ''' Allow the user to change the available columns by activating the ColumnHeaderContextMenu
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Allow the user to change the available columns by " & _
    "activating the ColumnHeaderContextMenu."), DefaultValue(True)> _
    Public Property AllowColumnChanges() As Boolean
        Get
            Return expLV.AllowColumnChanges
        End Get
        Set(ByVal Value As Boolean)
            expLV.AllowColumnChanges = Value
        End Set
    End Property

    ''' <summary>
    ''' Determines ExpList Background Image
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Determines ExpList Background Image."), _
    DefaultValue("")> _
    Public Shadows Property BackgroundImage() As String
        Get
            Return expLV.BackgroundImage
        End Get
        Set(ByVal Value As String)
            expLV.BackgroundImage = Value
        End Set
    End Property

    ''' <summary>
    ''' Determines ExpList Background Image Position
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Determines ExpList Background Image Position."), _
    DefaultValue(GetType(ContentAlignment), "MiddleCenter")> _
    Public Property BackgroundImagePosition() As ContentAlignment
        Get
            Return expLV.BackgroundImagePosition
        End Get
        Set(ByVal Value As ContentAlignment)
            expLV.BackgroundImagePosition = Value
        End Set
    End Property

    ''' <summary>
    ''' Determines ExpList View Style
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Determines ExpList View Style."), _
    DefaultValue(GetType(ExpListView), "LargeIcon")> _
    Public Property View() As ExpListView
        Get
            Return expLV.View
        End Get
        Set(ByVal Value As ExpListView)
            expLV.View = Value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the size of the list items in tile view
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("ExpList"), Description("Determines the size of the list items in tile view."), _
    DefaultValue(GetType(Size), "0,0")> _
    Public Property TileSize() As Size
        Get
            Return expLV.TileSize
        End Get
        Set(ByVal value As Size)
            expLV.TileSize = value
        End Set
    End Property

    ''' <summary>
    ''' Allow the user to rename files and folders in the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to rename files and folders in the control."), _
    DefaultValue(True)> _
        Public Property AllowRename() As Boolean
        Get
            Return expLV.AllowRename
        End Get
        Set(ByVal Value As Boolean)
            expLV.AllowRename = Value
        End Set
    End Property

    ''' <summary>
    ''' Allow the user to create new folders in the control
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Allow the user to create new folders in the control."), _
    DefaultValue(True)> _
        Public Property AllowNewFolders() As Boolean
        Get
            Return expLV.AllowNewFolders
        End Get
        Set(ByVal Value As Boolean)
            expLV.AllowNewFolders = Value
            btnNew.Enabled = Value
        End Set
    End Property

#End Region

    Private m_backList As New ArrayList             ' Array of previously selected items for navigation
    Private m_fowardList As New ArrayList           ' Array of items for reversing the navigation
    Private m_navigate As Boolean = False           ' Flag to see if user has clicked one of the back/foward buttons

    ''' <summary>
    ''' Determines if hidden files or folders should be shown
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("Behavior"), Description("Determines if hidden files or folders should be shown."), _
    DefaultValue(True)> _
    Public Property ShowHiddenFiles() As Boolean
        Get
            Return expLV.ShowHiddenFiles
        End Get
        Set(ByVal Value As Boolean)
            'set both ComboBox and ListView
            expLV.ShowHiddenFiles = Value
            expCB.ShowHiddenFolders = Value
        End Set
    End Property

    'Language for context menu and tool tips
    Private m_language As Language = Language.OperatingSystem
    ''' <summary>
    ''' Sets the language for the ContextMenu and ToolTips
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Browsable(False), DefaultValue(GetType(Language), "OperatingSystem")> _
    Public Property Language() As Language
        Get
            Return m_language
        End Get
        Set(ByVal Value As Language)
            If m_language = Value Then Exit Property
            m_language = Value
            Select Case Value
                Case Language.English
                    Me.expLV.Language = Language.English
                    Thread.CurrentThread.CurrentUICulture = New CultureInfo("en")
                    SetLanguage()
                    Refresh()
                Case Language.Deutsch
                    Me.expLV.Language = Language.Deutsch
                    Thread.CurrentThread.CurrentUICulture = New CultureInfo("de")
                    SetLanguage()
                    Refresh()
                Case Language.OperatingSystem
                    Me.expLV.Language = Language.OperatingSystem
                    Thread.CurrentThread.CurrentUICulture = CultureInfo.CurrentCulture
                    SetLanguage()
                    Refresh()
            End Select
        End Set
    End Property

#End Region

#Region " Windows Form Designer generated code "

    ''' <summary>
    ''' Creates a new instance of the Browser class
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call
        Me.btnBack.Image = My.Resources.Images.back
        Me.btnFoward.Image = My.Resources.Images.foward
        Me.btnNew.Image = My.Resources.Images.newFolder
        Me.btnUpOne.Image = My.Resources.Images.upOne
        Me.btnView.Image = My.Resources.Images.viewMenu
        mnuIcons.Image = My.Resources.Images.large_icon
        mnuTiles.Image = My.Resources.Images.tiles
        mnuThumbnails.Image = My.Resources.Images.thumbnails
        mnuDetails.Image = My.Resources.Images.details
        mnuList.Image = My.Resources.Images.list
        SetLanguage()
    End Sub

    'UserControl overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents expLV As ExpList
    Friend WithEvents expCB As ExpCombo
    Friend WithEvents viewMenu As System.Windows.Forms.ContextMenuStrip
    Friend WithEvents mnuIcons As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents mnuList As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents mnuDetails As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents mnuThumbnails As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents mnuTiles As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents pnlList As System.Windows.Forms.Panel
    Friend WithEvents tTip As System.Windows.Forms.ToolTip
    Friend WithEvents Label1 As System.Windows.Forms.Label
    Friend WithEvents btnUpOne As FlatButton
    Friend WithEvents btnBack As FlatButton
    Friend WithEvents btnFoward As FlatButton
    Friend WithEvents btnNew As FlatButton
    Friend WithEvents btnView As FlatButton
    Friend WithEvents pnlButtons As System.Windows.Forms.Panel
    Friend WithEvents pnlCombo As System.Windows.Forms.Panel
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.pnlList = New System.Windows.Forms.Panel
        Me.expLV = New ExplorerControls.ExpList
        Me.expCB = New ExplorerControls.ExpCombo
        Me.pnlCombo = New System.Windows.Forms.Panel
        Me.pnlButtons = New System.Windows.Forms.Panel
        Me.btnBack = New ExplorerControls.FlatButton
        Me.btnFoward = New ExplorerControls.FlatButton
        Me.btnUpOne = New ExplorerControls.FlatButton
        Me.Label1 = New System.Windows.Forms.Label
        Me.btnNew = New ExplorerControls.FlatButton
        Me.btnView = New ExplorerControls.FlatButton
        Me.viewMenu = New System.Windows.Forms.ContextMenuStrip(Me.components)
        Me.mnuIcons = New System.Windows.Forms.ToolStripMenuItem
        Me.mnuList = New System.Windows.Forms.ToolStripMenuItem
        Me.mnuDetails = New System.Windows.Forms.ToolStripMenuItem
        Me.mnuThumbnails = New System.Windows.Forms.ToolStripMenuItem
        Me.mnuTiles = New System.Windows.Forms.ToolStripMenuItem
        Me.tTip = New System.Windows.Forms.ToolTip(Me.components)
        Me.pnlList.SuspendLayout()
        Me.pnlCombo.SuspendLayout()
        Me.pnlButtons.SuspendLayout()
        Me.viewMenu.SuspendLayout()
        Me.SuspendLayout()
        '
        'pnlList
        '
        Me.pnlList.Controls.Add(Me.expLV)
        Me.pnlList.Controls.Add(Me.pnlCombo)
        Me.pnlList.Dock = System.Windows.Forms.DockStyle.Fill
        Me.pnlList.Location = New System.Drawing.Point(0, 0)
        Me.pnlList.Name = "pnlList"
        Me.pnlList.Size = New System.Drawing.Size(388, 400)
        Me.pnlList.TabIndex = 2
        Me.pnlList.TabStop = True
        '
        'expLV
        '
        Me.expLV.AllowColumnReorder = True
        Me.expLV.Dock = System.Windows.Forms.DockStyle.Fill
        Me.expLV.ExplorerCombo = Me.expCB
        Me.expLV.Location = New System.Drawing.Point(0, 24)
        Me.expLV.Name = "expLV"
        Me.expLV.Size = New System.Drawing.Size(388, 376)
        Me.expLV.TabIndex = 1
        Me.expLV.UseCompatibleStateImageBehavior = False
        '
        'expCB
        '
        Me.expCB.DisplayPath = False
        Me.expCB.Dock = System.Windows.Forms.DockStyle.Fill
        Me.expCB.Font = New System.Drawing.Font("Microsoft Sans Serif", 8.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
        Me.expCB.ItemHeight = 18
        Me.expCB.Location = New System.Drawing.Point(0, 0)
        Me.expCB.MaxDropDownItems = 14
        Me.expCB.Name = "expCB"
        Me.expCB.RootDirectory = ExplorerControls.ExpTree.StartDir.None
        Me.expCB.SelectedItem = Nothing
        Me.expCB.ShowHiddenFolders = True
        Me.expCB.Size = New System.Drawing.Size(248, 24)
        Me.expCB.StartUpDirectory = ExplorerControls.ExpTree.StartDir.None
        Me.expCB.TabIndex = 0
        '
        'pnlCombo
        '
        Me.pnlCombo.Controls.Add(Me.expCB)
        Me.pnlCombo.Controls.Add(Me.pnlButtons)
        Me.pnlCombo.Dock = System.Windows.Forms.DockStyle.Top
        Me.pnlCombo.Location = New System.Drawing.Point(0, 0)
        Me.pnlCombo.Name = "pnlCombo"
        Me.pnlCombo.Size = New System.Drawing.Size(388, 24)
        Me.pnlCombo.TabIndex = 0
        Me.pnlCombo.TabStop = True
        '
        'pnlButtons
        '
        Me.pnlButtons.Controls.Add(Me.btnBack)
        Me.pnlButtons.Controls.Add(Me.btnFoward)
        Me.pnlButtons.Controls.Add(Me.btnUpOne)
        Me.pnlButtons.Controls.Add(Me.Label1)
        Me.pnlButtons.Controls.Add(Me.btnNew)
        Me.pnlButtons.Controls.Add(Me.btnView)
        Me.pnlButtons.Dock = System.Windows.Forms.DockStyle.Right
        Me.pnlButtons.Location = New System.Drawing.Point(248, 0)
        Me.pnlButtons.Name = "pnlButtons"
        Me.pnlButtons.Size = New System.Drawing.Size(140, 24)
        Me.pnlButtons.TabIndex = 1
        Me.pnlButtons.TabStop = True
        '
        'btnBack
        '
        Me.btnBack.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.btnBack.ForeColor = System.Drawing.SystemColors.ControlText
        Me.btnBack.HighlightColor = System.Drawing.Color.Gainsboro
        Me.btnBack.HighlightHoverColor = System.Drawing.Color.DarkGray
        Me.btnBack.HoverColor = System.Drawing.Color.Silver
        Me.btnBack.Location = New System.Drawing.Point(0, 0)
        Me.btnBack.Name = "btnBack"
        Me.btnBack.Size = New System.Drawing.Size(23, 23)
        Me.btnBack.TabIndex = 0
        '
        'btnFoward
        '
        Me.btnFoward.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.btnFoward.ForeColor = System.Drawing.SystemColors.ControlText
        Me.btnFoward.HighlightColor = System.Drawing.Color.Gainsboro
        Me.btnFoward.HighlightHoverColor = System.Drawing.Color.DarkGray
        Me.btnFoward.HoverColor = System.Drawing.Color.Silver
        Me.btnFoward.Location = New System.Drawing.Point(24, 0)
        Me.btnFoward.Name = "btnFoward"
        Me.btnFoward.Size = New System.Drawing.Size(23, 23)
        Me.btnFoward.TabIndex = 1
        '
        'btnUpOne
        '
        Me.btnUpOne.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.btnUpOne.ForeColor = System.Drawing.SystemColors.ControlText
        Me.btnUpOne.HighlightColor = System.Drawing.Color.Gainsboro
        Me.btnUpOne.HighlightHoverColor = System.Drawing.Color.DarkGray
        Me.btnUpOne.HoverColor = System.Drawing.Color.Silver
        Me.btnUpOne.Location = New System.Drawing.Point(48, 0)
        Me.btnUpOne.Name = "btnUpOne"
        Me.btnUpOne.Size = New System.Drawing.Size(23, 23)
        Me.btnUpOne.TabIndex = 2
        '
        'Label1
        '
        Me.Label1.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.Label1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D
        Me.Label1.Location = New System.Drawing.Point(73, 2)
        Me.Label1.Name = "Label1"
        Me.Label1.Size = New System.Drawing.Size(2, 19)
        Me.Label1.TabIndex = 3
        '
        'btnNew
        '
        Me.btnNew.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.btnNew.ForeColor = System.Drawing.SystemColors.ControlText
        Me.btnNew.HighlightColor = System.Drawing.Color.Gainsboro
        Me.btnNew.HighlightHoverColor = System.Drawing.Color.DarkGray
        Me.btnNew.HoverColor = System.Drawing.Color.Silver
        Me.btnNew.Location = New System.Drawing.Point(75, 0)
        Me.btnNew.Name = "btnNew"
        Me.btnNew.Size = New System.Drawing.Size(23, 23)
        Me.btnNew.TabIndex = 4
        '
        'btnView
        '
        Me.btnView.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.btnView.ForeColor = System.Drawing.SystemColors.ControlText
        Me.btnView.HighlightColor = System.Drawing.Color.Gainsboro
        Me.btnView.HighlightHoverColor = System.Drawing.Color.DarkGray
        Me.btnView.HoverColor = System.Drawing.Color.Silver
        Me.btnView.Location = New System.Drawing.Point(99, 0)
        Me.btnView.Name = "btnView"
        Me.btnView.Size = New System.Drawing.Size(40, 23)
        Me.btnView.TabIndex = 4
        '
        'viewMenu
        '
        Me.viewMenu.Items.AddRange(New System.Windows.Forms.ToolStripItem() {Me.mnuIcons, Me.mnuList, Me.mnuDetails, Me.mnuThumbnails, Me.mnuTiles})
        Me.viewMenu.Name = "viewMenu"
        Me.viewMenu.Size = New System.Drawing.Size(153, 136)
        '
        'mnuIcons
        '
        Me.mnuIcons.Name = "mnuIcons"
        Me.mnuIcons.Size = New System.Drawing.Size(152, 22)
        Me.mnuIcons.Text = "&Icons"
        '
        'mnuList
        '
        Me.mnuList.Name = "mnuList"
        Me.mnuList.Size = New System.Drawing.Size(152, 22)
        Me.mnuList.Text = "&List"
        '
        'mnuDetails
        '
        Me.mnuDetails.Name = "mnuDetails"
        Me.mnuDetails.Size = New System.Drawing.Size(152, 22)
        Me.mnuDetails.Text = "&Details"
        '
        'mnuThumbnails
        '
        Me.mnuThumbnails.ImageAlign = System.Drawing.ContentAlignment.BottomRight
        Me.mnuThumbnails.Name = "mnuThumbnails"
        Me.mnuThumbnails.Size = New System.Drawing.Size(152, 22)
        Me.mnuThumbnails.Text = "Thumbnails"
        '
        'mnuTiles
        '
        Me.mnuTiles.Name = "mnuTiles"
        Me.mnuTiles.Size = New System.Drawing.Size(152, 22)
        Me.mnuTiles.Text = "&Tiles"
        '
        'Browser
        '
        Me.Controls.Add(Me.pnlList)
        Me.Name = "Browser"
        Me.Size = New System.Drawing.Size(388, 400)
        Me.pnlList.ResumeLayout(False)
        Me.pnlCombo.ResumeLayout(False)
        Me.pnlButtons.ResumeLayout(False)
        Me.viewMenu.ResumeLayout(False)
        Me.ResumeLayout(False)

    End Sub

#End Region

#Region " Set Strings "

    Private Sub SetLanguage()
        Me.tTip.SetToolTip(Me.btnBack, My.Resources.ControlStrings.Back)
        Me.tTip.SetToolTip(Me.btnFoward, My.Resources.ControlStrings.Foward)
        Me.tTip.SetToolTip(Me.btnUpOne, My.Resources.ControlStrings.UpOne)
        Me.tTip.SetToolTip(Me.btnNew, My.Resources.ControlStrings._New)
        Me.tTip.SetToolTip(Me.btnView, My.Resources.ControlStrings.View)
        Me.mnuDetails.Text = My.Resources.ControlStrings.Details
        Me.mnuList.Text = My.Resources.ControlStrings.List
        Me.mnuIcons.Text = My.Resources.ControlStrings.Icons
        Me.mnuThumbnails.Text = My.Resources.ControlStrings.Thumbnails
        Me.mnuTiles.Text = My.Resources.ControlStrings.Tiles
    End Sub

#End Region

#Region " Control Event Handlers "

    'Toggle view - Use Begin- and EndUpdate for flicker that occurs when the items are grouped
    Private Sub mnuIcons_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuIcons.Click
        Me.expLV.BeginUpdate()
        Me.expLV.View = ExpListView.LargeIcon
        Me.expLV.EndUpdate()
    End Sub

    Private Sub mnuList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuList.Click
        Me.expLV.BeginUpdate()
        Me.expLV.View = ExpListView.List
        Me.expLV.EndUpdate()
    End Sub

    Private Sub mnuDetails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDetails.Click
        Me.expLV.BeginUpdate()
        Me.expLV.View = ExpListView.Details
        Me.expLV.EndUpdate()
    End Sub

    Private Sub mnuThumbnails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuThumbnails.Click
        Me.expLV.BeginUpdate()
        Me.expLV.View = ExpListView.Thumbnail
        Me.expLV.EndUpdate()
    End Sub

    Private Sub mnuTiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTiles.Click
        Me.expLV.BeginUpdate()
        Me.View = ExpListView.Tile
        Me.expLV.EndUpdate()
    End Sub

    Private Sub btnNew_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnNew.Click
        expLV.CreateNewFolder()
    End Sub

    Private Sub btnView_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnView.Click
        Me.viewMenu.Show(btnView, New Point(0, btnView.Top + btnView.Height))
    End Sub

    Private Sub viewMenu_Opening(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles viewMenu.Opening
        'Uncheck all items
        mnuDetails.Checked = False
        mnuIcons.Checked = False
        mnuList.Checked = False
        mnuThumbnails.Checked = False
        mnuTiles.Checked = False
        'Check the selected item
        Select Case expLV.View
            Case ExpListView.Details
                mnuDetails.Checked = True
            Case ExpListView.LargeIcon
                mnuIcons.Checked = True
            Case ExpListView.List
                mnuList.Checked = True
            Case ExpListView.Thumbnail
                mnuThumbnails.Checked = True
            Case ExpListView.Tile
                mnuTiles.Checked = True
        End Select
    End Sub

    Private Sub expCB_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles expCB.SelectedIndexChanged
        If expCB.SelectedIndex < 0 Then Exit Sub

        'Manage Back/Foward Buttons
        If Not m_navigate Then
            'Don't add the same item twice
            If m_backList.Count = 0 Then
                m_backList.Add(expCB.SelectedItem)
            ElseIf Not m_backList(m_backList.Count - 1) Is expCB.SelectedItem Then
                m_backList.Add(expCB.SelectedItem)
            End If
            m_fowardList.Clear()
        End If
        CheckButtons()

        'Enable/Disable Up Button
        If Me.expCB.Items.IndexOf(expCB.SelectedItem) > 0 Then
            btnUpOne.Enabled = True
        Else
            btnUpOne.Enabled = False
        End If

        'Enable/Disable btnNew if a Special Folder or a
        'SharePoint Folder is selected (The IsLink is for
        'Network Places).
        Dim itm As CShItem = Me.expCB.SelectedItem.CShItem
        If (itm.IsLink) Or (itm.Path.StartsWith("::")) Or (itm.IsSharePoint) Then
            btnNew.Enabled = False
        Else
            btnNew.Enabled = expLV.AllowNewFolders
        End If

    End Sub

    Private Sub expLV_SelectedFolderChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles expLV.SelectedFolderChanged
        'Raise SelectedFolderChanged Event - see ExpList as
        'to why we have to do this here and not in the above
        'Event Handler
        OnSelectedFolderChanged(e)
    End Sub

    'Re-publish some ExpList Events...
    Private Sub expLV_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles expLV.SelectedIndexChanged
        OnSelectedListIndexChanged(e)
    End Sub

    Private Sub expLV_FileActivate(ByVal sender As Object, ByVal e As ExpList.ExpListItemEventArgs) Handles expLV.FileActivate
        OnFileActivate(e)
    End Sub

    Private Sub expLV_FolderActivate(ByVal sender As Object, ByVal e As ExpList.ExpListItemEventArgs) Handles expLV.FolderActivate
        OnFolderActivate(e)
    End Sub

    Private Sub expLV_AfterSort(ByVal sender As Object, ByVal e As ExpList.ExpListSortEventArgs) Handles expLV.AfterSort
        OnAfterSort(e)
    End Sub

    Private Sub expLV_BeforeSort(ByVal sender As Object, ByVal e As ExpList.ExpListSortEventArgs) Handles expLV.BeforeSort
        OnBeforeSort(e)
    End Sub

    Private Sub Browser_FontChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.FontChanged
        expCB.Font = Me.Font
        expLV.Font = Me.Font
        viewMenu.Font = Font
    End Sub

    Private Sub expLV_Group(ByVal sender As Object, ByVal e As ExpList.ExpListGroupEventArgs) Handles expLV.Group
        OnGroup(e)
    End Sub

    Private Sub expLV_ItemsAdded(ByVal sender As Object, ByVal e As ExpList.ExpListItemsAddedEventArgs) Handles expLV.ItemsAdded
        OnItemsAdded(e)
    End Sub

    Private Sub expLV_NewFolder(ByVal sender As Object, ByVal e As ExpList.ExpListItemEventArgs) Handles expLV.NewFolder
        OnNewFolder(e)
    End Sub

#End Region

#Region " Public Methods "

#Region " SortByColumn "

    ''' <summary>
    ''' Sorts the items in the ExpList
    ''' </summary>
    ''' <param name="column">The ExpListColumn to sort by</param>
    ''' <param name="order">The order to sort</param>
    ''' <remarks></remarks>
    Public Sub SortByColumn(ByVal column As ExpListColumn, ByVal order As Windows.Forms.SortOrder)
        expLV.SortByColumn(column, SortOrder)
    End Sub

#End Region

#Region " Saving and Loading State "

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <param name="xmlFile">Xml file for writing the DataSet to</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub SaveState(ByVal xmlFile As String)
        Dim ds As DataSet = SaveState()
        ds.WriteXml(xmlFile)
    End Sub

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <returns>DataSet with the current settings</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function SaveState() As DataSet

        'Create the dataset and tables
        Dim ds As New DataSet("BrowserState")
        'Table for the folders
        Dim dtFolders As New DataTable("SelectedFolder")
        Dim dcPath As New DataColumn("Path", System.Type.GetType("System.String"))

        dtFolders.Columns.Add(dcPath)
        ds.Tables.Add(dtFolders)

        'Store the Selected Path
        Dim row As DataRow
        row = dtFolders.NewRow
        If Not IsNothing(Me.SelectedComboItem) Then
            row.Item(0) = Me.SelectedComboItem.CShItem.Path
        Else
            row.Item(0) = ""
        End If
        dtFolders.Rows.Add(row)

        'Save the view state
        Dim dtView As New DataTable("View")
        Dim dcView As New DataColumn("ViewStyle")
        ds.Tables.Add(dtView)
        dtView.Columns.Add(dcView)

        row = dtView.NewRow
        row.Item(0) = Me.expLV.View
        dtView.Rows.Add(row)

        'Save the column width
        Dim dtColumns As New DataTable("Columns")
        Dim dcName As New DataColumn("Name")
        Dim dcIndex As New DataColumn("Index")
        Dim dcType As New DataColumn("Type")
        Dim dcWidth As New DataColumn("Width")

        ds.Tables.Add(dtColumns)
        dtColumns.Columns.Add(dcName)
        dtColumns.Columns.Add(dcIndex)
        dtColumns.Columns.Add(dcType)
        dtColumns.Columns.Add(dcWidth)

        Dim column As ExpListColumn
        Dim menuItem As ToolStripMenuItem
        For Each menuItem In Me.expLV.columnContextMenu.Items
            column = DirectCast(menuItem.Tag, ExpListColumn)
            If Not IsNothing(column) Then
                row = dtColumns.NewRow
                row.Item(0) = column.Name
                row.Item(1) = column.DisplayIndex
                row.Item(2) = CInt(column.ColumnType)
                row.Item(3) = column.Width
                dtColumns.Rows.Add(row)
            End If
        Next

        'Save the controls size
        Dim dtSize As New DataTable("Size")
        Dim dcX As New DataColumn("Width")
        Dim dcY As New DataColumn("Height")

        ds.Tables.Add(dtSize)
        dtSize.Columns.Add(dcX)
        dtSize.Columns.Add(dcY)

        row = dtSize.NewRow
        row.Item(0) = Me.Width
        row.Item(1) = Me.Height
        dtSize.Rows.Add(row)

        'Save the sorting column
        Dim dtSort As New DataTable("Sorting")
        Dim dcColumn As New DataColumn("ColumnIndex")
        Dim dcOrder As New DataColumn("Order")
        Dim dcGroup As New DataColumn("Group")

        ds.Tables.Add(dtSort)
        dtSort.Columns.Add(dcColumn)
        dtSort.Columns.Add(dcOrder)
        dtSort.Columns.Add(dcGroup)

        row = dtSort.NewRow
        If Not IsNothing(Me.SortColumn) Then
            row.Item(0) = Me.SortColumn.DisplayIndex
        Else
            row.Item(0) = -1
        End If
        row.Item(1) = Me.expLV.SortOrder
        row.Item(2) = Me.expLV.Grouped
        dtSort.Rows.Add(row)

        Return ds
    End Function

    ''' <summary>
    ''' Saves the current state of the control
    ''' </summary>
    ''' <param name="regKey">Registry Key for saving the state</param>
    ''' <remarks></remarks>
    Public Overridable Overloads Sub SaveState(ByVal regKey As RegistryKey)
        'Add the values to the reg key

        'Store the Selected Path
        Dim path As String
        If Not IsNothing(Me.SelectedComboItem) Then
            path = Me.SelectedComboItem.CShItem.Path
        Else
            path = ""
        End If
        regKey.SetValue("SelectedFolder", path)

        regKey.SetValue("View", Me.expLV.View)

        'Save the column width and type
        Dim columnsKey As RegistryKey = regKey.CreateSubKey("Columns")
        If Me.expLV.Columns.Count > 0 Then
            Dim column As ExpListColumn
            For Each column In Me.expLV.Columns
                columnsKey.SetValue(column.Name, New String() _
                        {CStr(column.DisplayIndex), CStr(column.ColumnType), _
                        CStr(column.Width)})
            Next
        End If

        'Save the controls size
        regKey.SetValue("Width", Me.Width)
        regKey.SetValue("Height", Me.Height)

        'Save the sorting column
        If Not IsNothing(Me.SortColumn) Then
            regKey.SetValue("SortingColumn", Me.SortColumn.DisplayIndex)
        Else
            regKey.SetValue("SortingColumn", -1)
        End If
        regKey.SetValue("SortOrder", Me.expLV.SortOrder)
        regKey.SetValue("Grouped", Me.expLV.Grouped)
    End Sub

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="xmlFile">Xml file with previously saved state information</param>
    ''' <returns>The size of the control when the state was saved</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function LoadState(ByVal xmlFile As String) As Size
        Try
            Dim ds As New DataSet
            ds.ReadXml(xmlFile)
            Return LoadState(ds)
        Catch ex As Exception
            Throw New ApplicationException("Settings file has an invalid format.", ex)
        End Try
    End Function

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="savedState">A DataSet with the previously saved state</param>
    ''' <returns>The size of the control when the state was saved</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function LoadState(ByVal savedState As DataSet) As Drawing.Size
        Return LoadState(savedState, False)
    End Function

    ''' <summary>
    ''' Loads a previously saved state
    ''' </summary>
    ''' <param name="savedState">A DataSet with the previously saved state</param>
    ''' <param name="throwOnError">Determines if any errors will be caught or if they are rethrown to the calling Class</param>
    ''' <returns>The size of the control when the state was saved</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function LoadState(ByVal savedState As DataSet, ByVal throwOnError As Boolean) As Size

        If IsNothing(savedState) Then Exit Function

        'Try to convert the data
        Try
            'Set the view
            Select Case savedState.Tables("View").Rows(0)(0).ToString
                Case "Details"
                    Me.expLV.View = ExpListView.Details
                Case "LargeIcon"
                    Me.expLV.View = ExpListView.LargeIcon
                Case "List"
                    Me.expLV.View = ExpListView.List
                Case "SmallIcon"
                    Me.expLV.View = ExpListView.SmallIcon
                Case "Thumbnail"
                    Me.expLV.View = ExpListView.Thumbnail
                Case "Tiles"
                    Me.expLV.View = ExpListView.Tile
            End Select

            'Set the columns
            Dim i As Integer
            Dim dt As DataTable = savedState.Tables("Columns")
            Dim column As ExpListColumn
            Dim index As Integer
            Dim menuItem As ToolStripItem
            Dim columnOrder As New Hashtable
            For i = 0 To dt.Rows.Count - 1
                menuItem = Me.expLV.columnContextMenu.Items(i)
                column = DirectCast(menuItem.Tag, ExpListColumn)
                column.Visible = False
                index = CInt(dt.Rows(i)(1))
                If index > -1 Then
                    column.ColumnType = CType(dt.Rows(i)(2), ExpListColumnType)
                    column.Width = CInt(dt.Rows(i)(3))
                    columnOrder.Add(index, column)
                End If
            Next
            For i = 0 To columnOrder.Count - 1
                DirectCast(columnOrder(i), ExpListColumn).Visible = True
            Next

            'Rebuild the combo then open the previously selected folder
            If expCB.Items.Count = 0 Then
                expCB.BuildCombo(CShItem.GetDeskTop)
            Else
                expCB.BuildCombo(expCB.Items(0).CShItem)
            End If
            Dim path As String = CStr(savedState.Tables("SelectedFolder").Rows _
                    (savedState.Tables("SelectedFolder").Rows.Count - 1)(0))
            If Not path = "" Then
                If Not expCB.SelectFolder(path) Then
                    Throw New ApplicationException("SelectFolder failed with path: " & path)
                End If
            Else
                If Not expCB.SelectFolder(expCB.Items(0).CShItem) Then
                    Throw New ApplicationException("SelectFolder failed with path: " & path)
                End If
            End If

            'Get the sort order 
            Dim order As Windows.Forms.SortOrder
            Select Case savedState.Tables("Sorting").Rows(0)(1).ToString.ToLower
                Case "descending"
                    order = SortOrder.Descending
                Case Else
                    order = SortOrder.Ascending
            End Select
            i = CInt(savedState.Tables("Sorting").Rows(0)(0))
            If i > -1 Then
                expLV.SortByColumn(Me.Columns(i), order)
                'Regroup if necessary
                If CBool(savedState.Tables("Sorting").Rows(0)(2)) = True Then
                    expLV.Grouped = True
                End If
            End If
            Return New Drawing.Size(CInt(savedState.Tables("Size").Rows(0)("Width")), _
                        CInt(savedState.Tables("Size").Rows(0)("Height")))
        Catch ex As Exception
            If throwOnError Then Throw ex
            'Had some sort of problem. Just rebuild the combo normally
            Dim startFolder As ExpTree.StartDir = Me.expCB.StartUpDirectory
            Me.expCB.RootDirectory = Me.expCB.RootDirectory
            Me.expCB.StartUpDirectory = startFolder
            Return New Drawing.Size(100, 100)
        End Try
    End Function

    ''' <summary>
    ''' Loads a previously saved state (will rebuild the control if an error occurs)
    ''' </summary>
    ''' <param name="savedState">A Registry Key with the previously saved state</param>
    ''' <returns>The size of the control when the state was saved</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function LoadState(ByVal savedState As RegistryKey) As Drawing.Size
        Return LoadState(savedState, False)
    End Function

    ''' <summary>
    ''' Loads a previously saved state
    ''' </summary>
    ''' <param name="savedState">A Registry Key with the previously saved state</param>
    ''' <param name="throwOnError">Determines if any errors will be caught or if they are rethrown to the calling Class</param>
    ''' <returns>The size of the control when the state was saved</returns>
    ''' <remarks></remarks>
    Public Overridable Overloads Function LoadState(ByVal savedState As RegistryKey, ByVal throwOnError As Boolean) As Drawing.Size

        If IsNothing(savedState) Then Exit Function

        'Try to convert the data
        Try
            'Set the view
            Select Case savedState.GetValue("View", "Details").ToString
                Case "Details"
                    Me.expLV.View = ExpListView.Details
                Case "LargeIcon"
                    Me.expLV.View = ExpListView.LargeIcon
                Case "List"
                    Me.expLV.View = ExpListView.List
                Case "SmallIcon"
                    Me.expLV.View = ExpListView.SmallIcon
                Case "Thumbnail"
                    Me.expLV.View = ExpListView.Thumbnail
                Case "Tiles"
                    Me.expLV.View = ExpListView.Tile
            End Select

            'Set the column width and type
            Dim i As Integer
            Dim columnsKey As RegistryKey = savedState.OpenSubKey("Columns")
            Dim column As ExpListColumn
            Dim index As Integer
            Dim menuItem As ToolStripItem
            Dim columnOrder As New Hashtable
            For i = 0 To Me.expLV.columnContextMenu.Items.Count - 1
                menuItem = Me.expLV.columnContextMenu.Items(i)
                column = DirectCast(menuItem.Tag, ExpListColumn)
                Dim values() As String = DirectCast(columnsKey.GetValue(column.Name), String())
                If IsNothing(values) Then Continue For
                If values.Length < 1 Then Continue For
                column.Visible = False
                index = CInt(values(0))
                If index > -1 Then
                    column.ColumnType = CType(values(1), ExpListColumnType)
                    column.Width = CInt(values(2))
                    columnOrder.Add(index, column)
                End If
            Next
            For i = 0 To columnOrder.Count - 1
                DirectCast(columnOrder(i), ExpListColumn).Visible = True
            Next

            'Rebuild the combo then open the previously selected folder
            If expCB.Items.Count = 0 Then
                expCB.BuildCombo(CShItem.GetDeskTop)
            Else
                expCB.BuildCombo(expCB.Items(0).CShItem)
            End If
            Dim path As String = CStr(savedState.GetValue("SelectedFolder", ""))
            If Not path = "" Then
                expCB.SelectFolder(path)
            End If

            'Get the sort order 
            Dim order As Windows.Forms.SortOrder
            Select Case CStr(savedState.GetValue("SortOrder", "ascending")).ToLower
                Case "descending"
                    order = SortOrder.Descending
                Case Else
                    order = SortOrder.Ascending
            End Select
            i = CInt(savedState.GetValue("SortingColumn", 0))

            If i > -1 Then
                expLV.SortByColumn(Me.Columns(i), order)
                'Regroup if necessary
                If CBool(savedState.GetValue("Grouped", False)) = True Then
                    expLV.Grouped = True
                End If
            End If
            Return New Drawing.Size(CInt(savedState.GetValue("Width", 250)), CInt(savedState.GetValue("Height", 250)))
        Catch ex As Exception
            If throwOnError Then Throw ex
            'Had some sort of problem. Just rebuild the combo normally
            Dim startFolder As ExpTree.StartDir = Me.expCB.StartUpDirectory
            Me.expCB.RootDirectory = Me.expCB.RootDirectory
            Me.expCB.StartUpDirectory = startFolder
            Return New Drawing.Size(100, 100)
        End Try
    End Function

#End Region

#Region " SelectFolder and BuildCombo "

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="startDir">The member of the StartDir enumeration to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal startDir As ExpTree.StartDir) As Boolean
        Return Me.expCB.SelectFolder(startDir)
    End Function

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="item">The CShItem to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal item As CShItem) As Boolean
        Return Me.expCB.SelectFolder(item)
    End Function

    ''' <summary>
    ''' Selects a folder in the ExpCombo
    ''' </summary>
    ''' <param name="path">The path of the folder to be selected</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overloads Function SelectFolder(ByVal path As String) As Boolean
        Return Me.expCB.SelectFolder(path)
    End Function

    ''' <summary>
    ''' Builds the ExpCombo with the specified root item, adding and sorting all the subfolders of that item
    ''' </summary>
    ''' <param name="rootItem">The root item for the ExpCombo</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overridable Function BuildCombo(ByVal rootItem As CShItem) As Boolean
        expCB.BuildCombo(rootItem, True)
    End Function

    ''' <summary>
    ''' Builds the ExpCombo with the specified root item, adding all the subfolders of that item
    ''' </summary>
    ''' <param name="rootItem">The root item for the ExpCombo</param>
    ''' <param name="sort">Specifies whether the subfolders should be sorted</param>
    ''' <returns>Boolean signifying success or failure</returns>
    ''' <remarks></remarks>
    Public Overridable Function BuildCombo(ByVal rootItem As CShItem, ByVal sort As Boolean) As Boolean
        expCB.BuildCombo(rootItem, sort)
        'Clear navigation History
        m_backList = New ArrayList
        m_fowardList = New ArrayList
        CheckButtons()
    End Function

#End Region

#Region " Navigation "

    ''' <summary>
    ''' Navigates to the parent folder
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub UpOne()
        'Navigate to parent folder
        Dim parent As ExpComboItem = expCB.SelectedItem.Parent
        expCB.SelectedIndex = expCB.Items.IndexOf(parent)
    End Sub

    ''' <summary>
    ''' Navigates fowards
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub Foward()
        'Navigate foward
        If m_fowardList.Count > 0 Then
            Dim itm As ExpComboItem = DirectCast(m_fowardList(m_fowardList.Count - 1), ExpComboItem)
            'Navigate fowards
            m_navigate = True
            If Me.expCB.Items.Contains(itm) Then
                Me.expCB.SelectedItem = itm
                'Remove the last item in the list and add it to the back list
                m_fowardList.Remove(itm)
                m_backList.Add(itm)
            Else
                m_fowardList.Clear()
            End If
            CheckButtons()
            m_navigate = False
        End If
    End Sub

    ''' <summary>
    ''' Navigates to the previous folder
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub Back()
        'Navigate back
        If (Not expCB.SelectedIndex < 0) And m_backList.Count > 0 Then
            Dim current As ExpComboItem = DirectCast(m_backList(m_backList.Count - 1), ExpComboItem)
            Dim previous As ExpComboItem = DirectCast(m_backList.Item(m_backList.Count - 2), ExpComboItem)
            m_navigate = True
            'Navigate back
            If Me.expCB.Items.Contains(previous) Then
                Me.expCB.SelectedItem = previous
                'Remove the last item in the list (the folder that was selected before the method call) 
                'and add it to the foward list
                m_backList.Remove(current)
                m_fowardList.Add(current)
            Else
                m_backList.Clear()
            End If
            CheckButtons()
            m_navigate = False
        End If
    End Sub

    Private Sub CheckButtons()
        'Enables/Disables Back/Foward buttons
        If m_fowardList.Count < 1 Then
            btnFoward.Enabled = False
        Else
            btnFoward.Enabled = True
        End If

        If m_backList.Count < 2 Then
            btnBack.Enabled = False
        Else
            btnBack.Enabled = True
        End If
    End Sub

    Private Sub btnBack_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnBack.Click
        Back()
        'Click
        Sounds.Click()
    End Sub

    Private Sub btnFoward_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnFoward.Click
        Foward()
        'Click
        Sounds.Click()
    End Sub

    Private Sub btnUpOne_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnUpOne.Click
        UpOne()
        'Click
        Sounds.Click()
    End Sub

#End Region

#Region " Refresh "

    ''' <summary>
    ''' Refreshes the currently selected folder
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub RefreshBrowser()
        expLV.RefreshList()
    End Sub

#End Region

#End Region

#Region " Protected Event Raising Methods "

    ''' <summary>
    ''' Raises the FileActivate Event when an ExpListItem that represents a file is activated
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnFileActivate(ByVal e As ExpList.ExpListItemEventArgs)
        RaiseEvent FileActivate(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the FolderActivate Event when an ExpListItem that represents a folder is activated
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnFolderActivate(ByVal e As ExpList.ExpListItemEventArgs)
        RaiseEvent FolderActivate(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the BeforeSort Event directly before the ExpList is sorted
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnBeforeSort(ByVal e As ExpList.ExpListSortEventArgs)
        RaiseEvent BeforeSort(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the AfterSort Event directly after the ExpList is sorted
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnAfterSort(ByVal e As ExpList.ExpListSortEventArgs)
        RaiseEvent AfterSort(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the SelectedFolderChanged Event when the selected item in the ExpCombo changes
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnSelectedFolderChanged(ByVal e As EventArgs)
        RaiseEvent SelectedFolderChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the SelectedListIndexChanged Event when the selected index in the ExpList changes
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnSelectedListIndexChanged(ByVal e As EventArgs)
        RaiseEvent SelectedListIndexChanged(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the Group Event directly before the ExpList is grouped
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnGroup(ByVal e As ExpList.ExpListGroupEventArgs)
        RaiseEvent Group(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the ItemsAdded Event directly after new Items have been added to the ExpList
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Friend Overridable Sub OnItemsAdded(ByVal e As ExpList.ExpListItemsAddedEventArgs)
        RaiseEvent ItemsAdded(Me, e)
    End Sub

    ''' <summary>
    ''' Raises the NewFolder Event when a new folder is created in the ExpList
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overridable Sub OnNewFolder(ByVal e As ExpList.ExpListItemEventArgs)
        RaiseEvent NewFolder(Me, e)
    End Sub

#End Region

End Class
